# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import copy
from unittest import mock

import fixtures
from neutronclient.common import exceptions as neutron_exception

from ec2api.api import vpc as vpc_api
from ec2api.tests.unit import base
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers
from ec2api.tests.unit import tools


class VpcTestCase(base.ApiTestCase):

    @mock.patch('ec2api.api.vpc._create_vpc')
    def test_create_vpc(self, create_vpc):
        create_vpc.return_value = fakes.DB_VPC_1

        def check_response(response):
            self.assertIn('vpc', response)
            vpc = resp['vpc']
            self.assertThat(fakes.EC2_VPC_1, matchers.DictMatches(vpc))
            create_vpc.assert_called_once_with(mock.ANY, fakes.CIDR_VPC_1)

            create_vpc.reset_mock()

        resp = self.execute('CreateVpc', {'CidrBlock': fakes.CIDR_VPC_1})
        check_response(resp)

        resp = self.execute('CreateVpc', {'CidrBlock': fakes.CIDR_VPC_1,
                                          'instanceTenancy': 'default'})
        check_response(resp)

    def test_create_vpc_invalid_cidr(self):
        self.neutron.create_router.side_effect = (
            tools.get_neutron_create('router', fakes.ID_OS_ROUTER_1))
        self.db_api.add_item.side_effect = tools.get_db_api_add_item(
            fakes.ID_EC2_VPC_1)

        def do_check(args, error_code):
            self.assert_execution_error(error_code, 'CreateVpc', args)
            self.assertEqual(0, self.neutron.create_router.call_count)

            self.neutron.reset_mock()
            self.db_api.reset_mock()

        do_check({'CidrBlock': 'bad_cidr'}, 'InvalidParameterValue')
        do_check({'CidrBlock': '10.0.0.0/8'}, 'InvalidVpc.Range')

    def test_create_vpc_overlimit(self):
        self.neutron.create_router.side_effect = (
            neutron_exception.OverQuotaClient)
        self.db_api.add_item.side_effect = tools.get_db_api_add_item(
            fakes.ID_EC2_VPC_1)

        self.assert_execution_error('VpcLimitExceeded', 'CreateVpc',
                                    {'CidrBlock': fakes.CIDR_VPC_1})
        self.neutron.create_router.assert_called_with({'router': {}})
        self.assertEqual(0, self.db_api.add_item.call_count)

    def test_delete_vpc(self):
        self.set_mock_db_items(fakes.DB_VPC_1, fakes.DB_ROUTE_TABLE_1,
                               fakes.DB_SECURITY_GROUP_1)

        resp = self.execute('DeleteVpc', {'VpcId': fakes.ID_EC2_VPC_1})

        self.assertEqual(True, resp['return'])
        self.neutron.delete_router.assert_called_once_with(
            fakes.ID_OS_ROUTER_1)
        self.db_api.delete_item.assert_any_call(
            mock.ANY,
            fakes.ID_EC2_VPC_1)
        self.db_api.delete_item.assert_any_call(
            mock.ANY,
            fakes.ID_EC2_ROUTE_TABLE_1)
        self.db_api.delete_item.assert_any_call(
            mock.ANY,
            fakes.ID_EC2_SECURITY_GROUP_1)

    def test_delete_vpc_not_found(self):
        self.set_mock_db_items()

        self.assert_execution_error('InvalidVpcID.NotFound', 'DeleteVpc',
                                    {'VpcId': fakes.ID_EC2_VPC_1})
        self.assertEqual(0, self.neutron.delete_router.call_count)
        self.assertEqual(0, self.db_api.delete_item.call_count)

    def test_delete_vpc_dependency_violation(self):
        def do_check():
            self.assert_execution_error('DependencyViolation', 'DeleteVpc',
                                        {'VpcId': fakes.ID_EC2_VPC_1})
            self.assertEqual(0, self.neutron.delete_router.call_count)
            self.assertEqual(0, self.db_api.delete_item.call_count)

            self.neutron.reset_mock()
            self.db_api.reset_mock()

        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1)]})
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1,
                               fakes.DB_IGW_1, fakes.DB_VPC_1, )
        do_check()

        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1)]})
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1,
                               fakes.DB_ROUTE_TABLE_1, fakes.DB_ROUTE_TABLE_2,
                               fakes.DB_VPC_1)
        do_check()

        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1,
                               fakes.DB_SECURITY_GROUP_2, fakes.DB_VPC_1)
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1),
                                 fakes.OS_SECURITY_GROUP_2]})
        do_check()

        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1)]})
        self.set_mock_db_items(fakes.DB_SECURITY_GROUP_1,
                               fakes.DB_VPN_GATEWAY_1, fakes.DB_VPC_1, )
        do_check()

    def test_delete_vpc_not_conststent_os_vpc(self):
        self.set_mock_db_items(fakes.DB_VPC_1, fakes.DB_ROUTE_TABLE_1)

        def check_response(resp):
            self.assertEqual(True, resp['return'])
            self.neutron.delete_router.assert_called_once_with(
                fakes.ID_OS_ROUTER_1)
            self.db_api.delete_item.assert_any_call(
                mock.ANY,
                fakes.ID_EC2_VPC_1)
            self.db_api.delete_item.assert_any_call(
                mock.ANY,
                fakes.ID_EC2_ROUTE_TABLE_1)

            self.neutron.reset_mock()
            self.db_api.reset_mock()

        self.neutron.delete_router.side_effect = neutron_exception.NotFound
        resp = self.execute('DeleteVpc', {'VpcId': fakes.ID_EC2_VPC_1})
        check_response(resp)

        self.neutron.delete_router.side_effect = neutron_exception.Conflict
        resp = self.execute('DeleteVpc', {'VpcId': fakes.ID_EC2_VPC_1})
        check_response(resp)

    @tools.screen_unexpected_exception_logs
    def test_delete_vpc_rollback(self):
        self.set_mock_db_items(fakes.DB_VPC_1, fakes.DB_ROUTE_TABLE_1)
        self.neutron.delete_router.side_effect = Exception()

        self.assert_execution_error(self.ANY_EXECUTE_ERROR, 'DeleteVpc',
                                    {'VpcId': fakes.ID_EC2_VPC_1})

        self.db_api.restore_item.assert_any_call(
            mock.ANY, 'vpc', fakes.DB_VPC_1)
        self.db_api.restore_item.assert_any_call(
            mock.ANY, 'rtb', fakes.DB_ROUTE_TABLE_1)

    def test_describe_vpcs(self):
        self.neutron.list_routers.return_value = (
            {'routers': [fakes.OS_ROUTER_DEFAULT,
                         fakes.OS_ROUTER_1, fakes.OS_ROUTER_2]})
        self.set_mock_db_items(fakes.DB_VPC_DEFAULT,
                               fakes.DB_VPC_1, fakes.DB_VPC_2)

        resp = self.execute('DescribeVpcs', {})
        self.assertThat(resp['vpcSet'],
                        matchers.ListMatches([fakes.EC2_VPC_DEFAULT,
                                              fakes.EC2_VPC_1,
                                              fakes.EC2_VPC_2]))

        resp = self.execute('DescribeVpcs',
                            {'VpcId.1': fakes.ID_EC2_VPC_1})
        self.assertThat(resp['vpcSet'],
                        matchers.ListMatches([fakes.EC2_VPC_1]))
        self.db_api.get_items_by_ids.assert_called_once_with(
            mock.ANY, set([fakes.ID_EC2_VPC_1]))

        self.check_filtering(
            'DescribeVpcs', 'vpcSet',
            [('cidr', fakes.CIDR_VPC_1),
             ('dhcp-options-id', 'default'),
             ('is-default', False),
             ('state', 'available'),
             ('vpc-id', fakes.ID_EC2_VPC_1)])
        self.check_tag_support(
            'DescribeVpcs', 'vpcSet',
            fakes.ID_EC2_VPC_1, 'vpcId')

    def test_describe_vpcs_no_router(self):
        self.neutron.list_routers.return_value = {'routers': []}
        self.set_mock_db_items(fakes.DB_VPC_DEFAULT, fakes.DB_VPC_1)

        resp = self.execute('DescribeVpcs', {})

        self.assertThat(resp['vpcSet'],
                        matchers.ListMatches([fakes.EC2_VPC_DEFAULT,
                                              fakes.EC2_VPC_1]))

    @mock.patch('ec2api.api.vpc._check_and_create_default_vpc')
    def test_describe_vpcs_no_default_vpc(self, check_and_create):
        def mock_check_and_create(context):
            self.set_mock_db_items(fakes.DB_VPC_DEFAULT)
        check_and_create.side_effect = mock_check_and_create

        resp = self.execute('DescribeVpcs', {})
        self.assertEqual(resp['vpcSet'], [fakes.EC2_VPC_DEFAULT])

        check_and_create.assert_called_once_with(mock.ANY)

    def test_describe_vpcs_with_default_vpc(self):
        self.set_mock_db_items(fakes.DB_VPC_DEFAULT)

        resp = self.execute('DescribeVpcs', {})
        self.assertEqual(resp['vpcSet'], [fakes.EC2_VPC_DEFAULT])

        self.db_api.add_item.assert_not_called()


class VpcPrivateTestCase(base.BaseTestCase):

    def setUp(self):
        super(VpcPrivateTestCase, self).setUp()
        self.context = base.create_context()
        self.nova, self.nova_admin = self.mock_nova()
        self.neutron = self.mock_neutron()
        self.db_api = self.mock_db()

    @mock.patch('ec2api.api.route_table.create_route')
    @mock.patch('ec2api.api.subnet.create_subnet')
    @mock.patch('ec2api.api.internet_gateway.attach_internet_gateway')
    @mock.patch('ec2api.api.internet_gateway.create_internet_gateway')
    @mock.patch('ec2api.api.security_group._create_default_security_group')
    @mock.patch('ec2api.api.route_table._create_route_table')
    def test_create_vpc(self, create_route_table,
                        create_default_security_group,
                        create_internet_gateway, attach_internet_gateway,
                        create_subnet, create_route):
        def _prepare_and_check(vpc=None, ec2_vpc=None,
                               route_table=None):
            self.neutron.create_router.side_effect = (
                tools.get_neutron_create('router', vpc['os_id']))
            self.db_api.add_item.side_effect = (
                tools.get_db_api_add_item({'vpc': vpc['id']}))
            self.db_api.set_mock_items(vpc)
            create_route_table.return_value = route_table

            resp = vpc_api._create_vpc(self.context, vpc['cidr_block'],
                                       vpc['is_default'])

            # Check creation of vpc
            self.neutron.create_router.assert_called_with({'router': {}})
            self.neutron.update_router.assert_called_once_with(
                vpc['os_id'],
                {'router': {'name': ec2_vpc['vpcId']}})
            self.db_api.add_item.assert_called_once_with(
                mock.ANY, 'vpc', tools.purge_dict(
                    vpc, ('id', 'vpc_id', 'route_table_id')))
            self.db_api.update_item.assert_called_once_with(
                mock.ANY, vpc)

            create_route_table.assert_called_once_with(
                mock.ANY, vpc)
            create_default_security_group.assert_called_once_with(
                mock.ANY, vpc)

        _prepare_and_check(vpc=fakes.DB_VPC_1, ec2_vpc=fakes.EC2_VPC_1,
                           route_table=fakes.DB_ROUTE_TABLE_1)

        # checking that no default vpc related stuff is added
        create_internet_gateway.assert_not_called()
        attach_internet_gateway.assert_not_called()
        create_subnet.assert_not_called()
        create_route.assert_not_called()

        self.neutron.reset_mock()
        self.db_api.reset_mock()
        create_route_table.reset_mock()
        create_default_security_group.reset_mock()

        # Creation of default vpc
        create_route_table.return_value = fakes.DB_ROUTE_TABLE_DEFAULT
        create_subnet.return_value = {'subnet': fakes.EC2_SUBNET_DEFAULT}
        create_internet_gateway.return_value = {'internetGateway':
                                                fakes.EC2_IGW_DEFAULT}

        _prepare_and_check(vpc=fakes.DB_VPC_DEFAULT,
                           ec2_vpc=fakes.EC2_VPC_DEFAULT,
                           route_table=fakes.DB_ROUTE_TABLE_DEFAULT)

        create_internet_gateway.assert_called_once_with(mock.ANY)
        attach_internet_gateway.assert_called_once_with(mock.ANY,
            fakes.ID_EC2_IGW_DEFAULT,
            fakes.ID_EC2_VPC_DEFAULT)
        create_subnet.assert_called_once_with(mock.ANY,
            fakes.ID_EC2_VPC_DEFAULT,
            fakes.CIDR_SUBNET_DEFAULT)
        create_route.assert_called_once_with(mock.ANY,
            fakes.ID_EC2_ROUTE_TABLE_DEFAULT,
            '0.0.0.0/0', gateway_id=fakes.ID_EC2_IGW_DEFAULT)

    @mock.patch('ec2api.api.vpc._create_vpc')
    def test_check_and_create_default_vpc(self, create_vpc):
        self.configure(disable_ec2_classic=True)
        vpc_api._check_and_create_default_vpc(self.context)

        create_vpc.assert_called_once_with(mock.ANY, fakes.CIDR_VPC_DEFAULT,
                                           is_default=True)

    @tools.screen_logs('ec2api.api.vpc')
    @mock.patch('ec2api.api.internet_gateway.detach_internet_gateway')
    @mock.patch('ec2api.api.route_table.create_route')
    @mock.patch('ec2api.api.subnet.create_subnet')
    @mock.patch('ec2api.api.internet_gateway.attach_internet_gateway')
    @mock.patch('ec2api.api.internet_gateway.create_internet_gateway')
    @mock.patch('ec2api.api.security_group._create_default_security_group')
    @mock.patch('ec2api.api.route_table._create_route_table')
    def test_create_vpc_rollback(self, create_route_table,
                                 create_default_security_group,
                                 create_internet_gateway,
                                 attach_internet_gateway, create_subnet,
                                 create_route, detach_internet_gateway):
        self.configure(disable_ec2_classic=True)

        self.neutron.create_router.side_effect = (
            tools.get_neutron_create('router', fakes.ID_OS_ROUTER_DEFAULT))

        self.db_api.add_item.side_effect = (
            tools.get_db_api_add_item({'vpc': fakes.ID_EC2_VPC_DEFAULT}))

        DB_IGW_DEFAULT_DETACHED = (
            {'id': fakes.ID_EC2_IGW_DEFAULT,
            'os_id': None,
            'vpc_id': None})
        self.db_api.get_item_by_id.side_effect = (
            tools.get_db_api_get_item_by_id(fakes.DB_VPC_DEFAULT,
                                            fakes.DB_SUBNET_DEFAULT,
                                            fakes.DB_SECURITY_GROUP_DEFAULT,
                                            DB_IGW_DEFAULT_DETACHED))
        create_route_table.return_value = fakes.DB_ROUTE_TABLE_DEFAULT
        create_internet_gateway.return_value = {'internetGateway':
                                                fakes.EC2_IGW_DEFAULT}
        create_subnet.return_value = {'subnet': fakes.EC2_SUBNET_DEFAULT}
        create_default_security_group.return_value = (
            fakes.ID_EC2_SECURITY_GROUP_DEFAULT)

        # exception during attaching internet gateway
        create_route.side_effect = Exception()

        vpc_api._check_and_create_default_vpc(self.context)

        detach_internet_gateway.assert_any_call(mock.ANY,
                                                fakes.ID_EC2_IGW_DEFAULT,
                                                fakes.ID_EC2_VPC_DEFAULT)
        self.db_api.delete_item.assert_any_call(mock.ANY,
            fakes.ID_EC2_SUBNET_DEFAULT)
        self.db_api.delete_item.assert_any_call(mock.ANY,
            fakes.ID_EC2_IGW_DEFAULT)
        self.neutron.delete_security_group.assert_any_call(
            fakes.ID_OS_SECURITY_GROUP_DEFAULT)
        self.db_api.delete_item.assert_any_call(mock.ANY,
            fakes.ID_EC2_ROUTE_TABLE_DEFAULT)
        self.db_api.delete_item.assert_any_call(mock.ANY,
            fakes.ID_EC2_VPC_DEFAULT)
        self.neutron.delete_router.assert_called_once_with(
            fakes.ID_OS_ROUTER_DEFAULT)

    @mock.patch('ec2api.api.vpc._create_vpc')
    def test_check_and_create_default_vpc_failed(self, create_vpc):
        self.configure(disable_ec2_classic=True)
        create_vpc.side_effect = Exception()
        with fixtures.LoggerFixture(
                format='[%(levelname)s] %(message)s') as log:
            vpc_api._check_and_create_default_vpc(self.context)
        self.assertTrue(log.output.startswith(
            '[ERROR] Failed to create default vpc'))



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import importlib
from unittest import mock

from oslo_config import cfg
from oslo_config import fixture as config_fixture
from oslo_context import context
from oslotest import base as test_base

from ec2api import clients
from ec2api import context as ec2_context


GROUP_AUTHTOKEN = 'keystone_authtoken'


class ContextTestCase(test_base.BaseTestCase):

    @mock.patch('keystoneauth1.loading.load_auth_from_conf_options')
    @mock.patch('keystoneauth1.loading.load_session_from_conf_options')
    def test_get_os_admin_context(self, session, auth):
        conf = config_fixture.Config()
        clients._admin_session = None
        conf.config(auth_type='fake', group=GROUP_AUTHTOKEN)

        importlib.reload(ec2_context)
        # NOTE(ft): initialize a regular context to populate oslo_context's
        # local storage to prevent admin context to populate it.
        # Used to implicitly validate overwrite=False argument of the call
        # RequestContext constructor from inside get_os_admin_context
        if not context.get_current():
            ec2_context.RequestContext(None, None)

        ctx = ec2_context.get_os_admin_context()
        conf = cfg.CONF
        auth.assert_called_once_with(conf, GROUP_AUTHTOKEN)
        auth_plugin = auth.return_value
        session.assert_called_once_with(conf, GROUP_AUTHTOKEN,
                                        auth=auth_plugin)
        self.assertIsNone(ctx.user_id)
        self.assertIsNone(ctx.project_id)
        self.assertIsNone(ctx.auth_token)
        self.assertEqual([], ctx.service_catalog)
        self.assertTrue(ctx.is_os_admin)
        self.assertIsNotNone(ctx.session)
        self.assertIsNotNone(ctx.session.auth)
        self.assertNotEqual(context.get_current(), ctx)

        session.reset_mock()
        ec2_context.get_os_admin_context()
        self.assertFalse(session.called)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import copy
from unittest import mock

from neutronclient.common import exceptions as neutron_exception

from ec2api.api import ec2utils
from ec2api.tests.unit import base
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers
from ec2api.tests.unit import tools


class NetworkInterfaceTestCase(base.ApiTestCase):

    def test_create_network_interface(self):
        self.set_mock_db_items(fakes.DB_SUBNET_1, fakes.DB_VPC_1,
                               fakes.DB_SECURITY_GROUP_1)
        self.db_api.add_item.return_value = fakes.DB_NETWORK_INTERFACE_1
        self.neutron.show_subnet.return_value = {'subnet': fakes.OS_SUBNET_1}
        self.neutron.create_port.return_value = {'port': fakes.OS_PORT_1}
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1)]})

        def check_response(resp, auto_ips=False):
            self.assertThat(fakes.EC2_NETWORK_INTERFACE_1,
                            matchers.DictMatches(resp['networkInterface']))
            self.db_api.add_item.assert_called_once_with(
                mock.ANY, 'eni',
                tools.purge_dict(fakes.DB_NETWORK_INTERFACE_1, ('id',)))
            if auto_ips:
                self.neutron.create_port.assert_called_once_with(
                    {'port':
                        {'network_id': fakes.ID_OS_NETWORK_1,
                         'fixed_ips':
                            [{'subnet_id': fakes.ID_OS_SUBNET_1}],
                         'security_groups': [fakes.ID_OS_SECURITY_GROUP_1]}})
            else:
                self.neutron.create_port.assert_called_once_with(
                    {'port':
                        {'network_id': fakes.ID_OS_NETWORK_1,
                         'fixed_ips':
                            [{'ip_address': fakes.IP_NETWORK_INTERFACE_1}],
                         'security_groups': [fakes.ID_OS_SECURITY_GROUP_1]}})
            self.neutron.update_port.assert_called_once_with(
                fakes.ID_OS_PORT_1,
                {'port': {'name':
                          fakes.ID_EC2_NETWORK_INTERFACE_1}})
            self.neutron.reset_mock()
            self.db_api.reset_mock()
            self.neutron.list_security_groups.return_value = (
                {'security_groups': [
                    copy.deepcopy(fakes.OS_SECURITY_GROUP_1)]})

        resp = self.execute(
            'CreateNetworkInterface',
            {'SubnetId': fakes.ID_EC2_SUBNET_1,
             'PrivateIpAddress': fakes.IP_NETWORK_INTERFACE_1,
             'Description': fakes.DESCRIPTION_NETWORK_INTERFACE_1})
        check_response(resp)

        resp = self.execute(
            'CreateNetworkInterface',
            {'SubnetId': fakes.ID_EC2_SUBNET_1,
             'PrivateIpAddresses.1.PrivateIpAddress': (
                    fakes.IP_NETWORK_INTERFACE_1),
             'PrivateIpAddresses.1.Primary': True,
             'Description': fakes.DESCRIPTION_NETWORK_INTERFACE_1})
        check_response(resp)

        resp = self.execute(
            'CreateNetworkInterface',
            {'SubnetId': fakes.ID_EC2_SUBNET_1,
             'Description': fakes.DESCRIPTION_NETWORK_INTERFACE_1})
        check_response(resp, True)

        resp = self.execute(
            'CreateNetworkInterface',
            {'SubnetId': fakes.ID_EC2_SUBNET_1,
             'SecondaryPrivateIpAddressCount': '1',
             'Description': fakes.DESCRIPTION_NETWORK_INTERFACE_1})
        check_response(resp, True)

        resp = self.execute(
            'CreateNetworkInterface',
            {'SubnetId': fakes.ID_EC2_SUBNET_1,
             'SecondaryPrivateIpAddressCount': '0',
             'Description': fakes.DESCRIPTION_NETWORK_INTERFACE_1})
        check_response(resp, True)

    def test_create_network_interface_multiple_ips(self):
        self.set_mock_db_items(fakes.DB_SUBNET_2, fakes.DB_VPC_1,
                               fakes.DB_SECURITY_GROUP_1)
        self.db_api.add_item.return_value = fakes.DB_NETWORK_INTERFACE_2
        self.neutron.show_subnet.return_value = {'subnet': fakes.OS_SUBNET_2}
        self.neutron.create_port.return_value = {'port': fakes.OS_PORT_2}
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1)]})
        created_ec2_network_interface = tools.patch_dict(
            fakes.EC2_NETWORK_INTERFACE_2,
            {'privateIpAddressesSet': [
                tools.purge_dict(s, ['association'])
                for s in fakes.EC2_NETWORK_INTERFACE_2[
                    'privateIpAddressesSet']]},
            ['association'])

        def check_response(resp):
            self.assertThat(created_ec2_network_interface,
                            matchers.DictMatches(resp['networkInterface']))
            self.db_api.add_item.assert_called_once_with(
                mock.ANY, 'eni',
                tools.purge_dict(fakes.DB_NETWORK_INTERFACE_2,
                                 ('id',
                                  'device_index',
                                  'instance_id',
                                  'delete_on_termination',
                                  'attach_time')))
            self.neutron.update_port.assert_called_once_with(
                fakes.ID_OS_PORT_2,
                {'port': {'name':
                          fakes.ID_EC2_NETWORK_INTERFACE_2}})
            self.neutron.reset_mock()
            self.db_api.reset_mock()
            self.neutron.list_security_groups.return_value = (
                {'security_groups': [
                    copy.deepcopy(fakes.OS_SECURITY_GROUP_1)]})

        resp = self.execute(
            'CreateNetworkInterface',
            {'SubnetId': fakes.ID_EC2_SUBNET_2,
             'SecondaryPrivateIpAddressCount': '3',
             'Description': fakes.DESCRIPTION_NETWORK_INTERFACE_2})
        self.neutron.create_port.assert_called_once_with(
            {'port': {'network_id': fakes.ID_OS_NETWORK_2,
                      'fixed_ips': [{'subnet_id': fakes.ID_OS_SUBNET_2},
                                    {'subnet_id': fakes.ID_OS_SUBNET_2},
                                    {'subnet_id': fakes.ID_OS_SUBNET_2}],
                      'security_groups': [fakes.ID_OS_SECURITY_GROUP_1]}})
        check_response(resp)

        resp = self.execute(
            'CreateNetworkInterface',
            {'SubnetId': fakes.ID_EC2_SUBNET_2,
             'PrivateIpAddress': fakes.IPS_NETWORK_INTERFACE_2[0],
             'PrivateIpAddresses.1.PrivateIpAddress':
                 fakes.IPS_NETWORK_INTERFACE_2[1],
             'PrivateIpAddresses.1.Primary': False,
             'PrivateIpAddresses.2.PrivateIpAddress':
                 fakes.IPS_NETWORK_INTERFACE_2[2],
             'PrivateIpAddresses.2.Primary': False,
             'Description': fakes.DESCRIPTION_NETWORK_INTERFACE_2})
        self.neutron.create_port.assert_called_once_with(
            {'port':
             {'network_id': fakes.ID_OS_NETWORK_2,
              'fixed_ips': [
                  {'ip_address': fakes.IPS_NETWORK_INTERFACE_2[0]},
                  {'ip_address': fakes.IPS_NETWORK_INTERFACE_2[1]},
                  {'ip_address': fakes.IPS_NETWORK_INTERFACE_2[2]}],
              'security_groups': [fakes.ID_OS_SECURITY_GROUP_1]}})
        check_response(resp)

        resp = self.execute(
            'CreateNetworkInterface',
            {'SubnetId': fakes.ID_EC2_SUBNET_2,
             'PrivateIpAddresses.1.PrivateIpAddress':
                 fakes.IPS_NETWORK_INTERFACE_2[0],
             'PrivateIpAddresses.1.Primary': True,
             'PrivateIpAddresses.2.PrivateIpAddress':
                 fakes.IPS_NETWORK_INTERFACE_2[1],
             'PrivateIpAddresses.2.Primary': False,
             'PrivateIpAddresses.3.PrivateIpAddress':
                 fakes.IPS_NETWORK_INTERFACE_2[2],
             'PrivateIpAddresses.3.Primary': False,
             'Description': fakes.DESCRIPTION_NETWORK_INTERFACE_2})
        self.neutron.create_port.assert_called_once_with(
            {'port':
             {'network_id': fakes.ID_OS_NETWORK_2,
              'fixed_ips': [
                  {'ip_address': fakes.IPS_NETWORK_INTERFACE_2[0]},
                  {'ip_address': fakes.IPS_NETWORK_INTERFACE_2[1]},
                  {'ip_address': fakes.IPS_NETWORK_INTERFACE_2[2]}],
              'security_groups': [fakes.ID_OS_SECURITY_GROUP_1]}})
        check_response(resp)

        resp = self.execute(
            'CreateNetworkInterface',
            {'SubnetId': fakes.ID_EC2_SUBNET_2,
             'PrivateIpAddress': fakes.IPS_NETWORK_INTERFACE_2[0],
             'PrivateIpAddresses.1.PrivateIpAddress':
                 fakes.IPS_NETWORK_INTERFACE_2[1],
             'PrivateIpAddresses.1.Primary': False,
             'SecondaryPrivateIpAddressCount': '1',
             'Description': fakes.DESCRIPTION_NETWORK_INTERFACE_2})
        self.neutron.create_port.assert_called_once_with(
            {'port':
             {'network_id': fakes.ID_OS_NETWORK_2,
              'fixed_ips': [
                  {'ip_address': fakes.IPS_NETWORK_INTERFACE_2[0]},
                  {'ip_address': fakes.IPS_NETWORK_INTERFACE_2[1]},
                  {'subnet_id': fakes.ID_OS_SUBNET_2}],
              'security_groups': [fakes.ID_OS_SECURITY_GROUP_1]}})
        check_response(resp)

    def test_create_network_interface_invalid_parameters(self):
        def do_check(args, error_code):
            self.neutron.reset_mock()
            self.db_api.reset_mock()
            self.assert_execution_error(
                error_code, 'CreateNetworkInterface', args)

        self.set_mock_db_items()
        do_check({'SubnetId': fakes.ID_EC2_SUBNET_2},
                 'InvalidSubnetID.NotFound')
        self.db_api.get_item_by_id.assert_called_once_with(
            mock.ANY, fakes.ID_EC2_SUBNET_2)

        self.set_mock_db_items(fakes.DB_SUBNET_1, fakes.DB_VPC_1)
        self.neutron.show_subnet.return_value = {'subnet': fakes.OS_SUBNET_1}
        do_check({'SubnetId': fakes.ID_EC2_SUBNET_1,
                  'PrivateIpAddress': fakes.IP_NETWORK_INTERFACE_2},
                 'InvalidParameterValue')

        for cls in [neutron_exception.OverQuotaClient,
                    neutron_exception.IpAddressGenerationFailureClient]:
            self.neutron.create_port.side_effect = cls()
            do_check({'SubnetId': fakes.ID_EC2_SUBNET_1,
                      'PrivateIpAddress': fakes.IP_NETWORK_INTERFACE_1},
                     'InsufficientFreeAddressesInSubnet')

        for cls in [neutron_exception.IpAddressInUseClient,
                    neutron_exception.BadRequest]:
            self.neutron.create_port.side_effect = cls()
            do_check({'SubnetId': fakes.ID_EC2_SUBNET_1,
                      'PrivateIpAddress': fakes.IP_NETWORK_INTERFACE_1},
                     'InvalidParameterValue')

    @tools.screen_unexpected_exception_logs
    @mock.patch('ec2api.api.dhcp_options._add_dhcp_opts_to_port')
    def test_create_network_interface_rollback(self, _add_dhcp_opts_to_port):
        self.set_mock_db_items(
            tools.update_dict(
                fakes.DB_VPC_1,
                {'dhcp_options_id': fakes.ID_EC2_DHCP_OPTIONS_1}),
            fakes.DB_SUBNET_1, fakes.DB_DHCP_OPTIONS_1)
        self.db_api.add_item.return_value = fakes.DB_NETWORK_INTERFACE_1
        self.neutron.show_subnet.return_value = {'subnet': fakes.OS_SUBNET_1}
        self.neutron.create_port.return_value = {'port': fakes.OS_PORT_1}
        _add_dhcp_opts_to_port.side_effect = Exception()

        self.assert_execution_error(self.ANY_EXECUTE_ERROR,
                                    'CreateNetworkInterface',
                                    {'SubnetId': fakes.ID_EC2_SUBNET_1})

        self.neutron.delete_port.assert_called_once_with(fakes.ID_OS_PORT_1)
        self.db_api.delete_item.assert_called_once_with(
            mock.ANY, fakes.ID_EC2_NETWORK_INTERFACE_1)

    def test_delete_network_interface(self):
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_1)
        resp = self.execute(
            'DeleteNetworkInterface',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1})
        self.assertEqual(True, resp['return'])
        self.db_api.get_item_by_id.assert_any_call(
            mock.ANY,
            fakes.ID_EC2_NETWORK_INTERFACE_1)
        self.db_api.delete_item.assert_called_once_with(
            mock.ANY,
            fakes.ID_EC2_NETWORK_INTERFACE_1)
        self.neutron.delete_port.assert_called_once_with(
            fakes.ID_OS_PORT_1)

    def test_delete_network_interface_obsolete(self):
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_1)
        self.neutron.delete_port.side_effect = (
            neutron_exception.PortNotFoundClient())
        resp = self.execute(
            'DeleteNetworkInterface',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1})
        self.assertEqual(True, resp['return'])

    def test_delete_network_interface_no_network_interface(self):
        self.set_mock_db_items()
        self.assert_execution_error(
            'InvalidNetworkInterfaceID.NotFound', 'DeleteNetworkInterface',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1})
        self.assertEqual(0, self.neutron.delete_port.call_count)

    def test_delete_network_interface_is_in_use(self):
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_2)
        self.assert_execution_error(
            'InvalidParameterValue', 'DeleteNetworkInterface',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2})
        self.assertEqual(0, self.neutron.delete_port.call_count)

    def test_delete_network_interface_with_public_ip(self):
        detached_network_interface_2 = fakes.gen_db_network_interface(
            fakes.ID_EC2_NETWORK_INTERFACE_2,
            fakes.ID_OS_PORT_2,
            fakes.ID_EC2_VPC_1,
            fakes.ID_EC2_SUBNET_2,
            fakes.IP_NETWORK_INTERFACE_2)
        self.set_mock_db_items(detached_network_interface_2,
                               fakes.DB_ADDRESS_1, fakes.DB_ADDRESS_2)
        resp = self.execute(
            'DeleteNetworkInterface',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2})
        self.assertEqual(True, resp['return'])
        self.db_api.get_item_by_id.assert_any_call(
            mock.ANY,
            fakes.ID_EC2_NETWORK_INTERFACE_2)
        self.db_api.delete_item.assert_called_once_with(
            mock.ANY,
            fakes.ID_EC2_NETWORK_INTERFACE_2)
        self.neutron.delete_port.assert_called_once_with(
            fakes.ID_OS_PORT_2)
        self.db_api.update_item.assert_called_once_with(
            mock.ANY,
            tools.purge_dict(fakes.DB_ADDRESS_2,
                             ['network_interface_id',
                              'private_ip_address']))

    @tools.screen_unexpected_exception_logs
    def test_delete_network_interface_rollback(self):
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_1)
        self.neutron.delete_port.side_effect = Exception()

        self.assert_execution_error(
            self.ANY_EXECUTE_ERROR, 'DeleteNetworkInterface',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1})

        self.db_api.restore_item.assert_called_once_with(
            mock.ANY, 'eni', fakes.DB_NETWORK_INTERFACE_1)

    def test_describe_network_interfaces(self):
        self.set_mock_db_items(
            fakes.DB_NETWORK_INTERFACE_1, fakes.DB_NETWORK_INTERFACE_2,
            fakes.DB_ADDRESS_1, fakes.DB_ADDRESS_2,
            fakes.DB_INSTANCE_1, fakes.DB_INSTANCE_2,
            fakes.DB_SECURITY_GROUP_1)
        self.neutron.list_ports.return_value = (
            {'ports': [fakes.OS_PORT_1, fakes.OS_PORT_2]})
        self.neutron.list_floatingips.return_value = (
            {'floatingips': [fakes.OS_FLOATING_IP_1,
                             fakes.OS_FLOATING_IP_2]})
        self.neutron.list_security_groups.return_value = (
            {'security_groups': [copy.deepcopy(fakes.OS_SECURITY_GROUP_1)]})

        resp = self.execute('DescribeNetworkInterfaces', {})
        self.assertThat(resp['networkInterfaceSet'],
                        matchers.ListMatches(
                            [fakes.EC2_NETWORK_INTERFACE_1,
                             fakes.EC2_NETWORK_INTERFACE_2],
                            orderless_lists=True),
                        verbose=True)

        self.db_api.get_items_by_ids = tools.CopyingMock(
            return_value=[fakes.DB_NETWORK_INTERFACE_1])
        resp = self.execute(
            'DescribeNetworkInterfaces',
            {'NetworkInterfaceId.1': fakes.ID_EC2_NETWORK_INTERFACE_1})
        self.assertThat(resp['networkInterfaceSet'],
                        matchers.ListMatches(
                            [fakes.EC2_NETWORK_INTERFACE_1]))
        self.db_api.get_items_by_ids.assert_called_once_with(
            mock.ANY, set([fakes.ID_EC2_NETWORK_INTERFACE_1]))

        self.check_filtering(
            'DescribeNetworkInterfaces', 'networkInterfaceSet',
            [('addresses.private-ip-address',
              fakes.IP_NETWORK_INTERFACE_2_EXT_1,),
             ('addresses.primary', False),
             ('addresses.association.public-ip', fakes.IP_ADDRESS_2),
             ('addresses.association.owner-id', fakes.ID_OS_PROJECT),
             ('association.association-id', fakes.ID_EC2_ASSOCIATION_2),
             ('association.allocation-id', fakes.ID_EC2_ADDRESS_2),
             ('association.ip-owner-id', fakes.ID_OS_PROJECT),
             ('association.public-ip', fakes.IP_ADDRESS_2),
             ('attachment.attachment-id',
              fakes.ID_EC2_NETWORK_INTERFACE_2_ATTACH),
             ('attachment.instance-id', fakes.ID_EC2_INSTANCE_1),
             ('attachment.instance-owner-id', fakes.ID_OS_PROJECT),
             ('attachment.device-index', 0),
             ('attachment.status', 'attached'),
             ('attachment.attach.time', fakes.TIME_ATTACH_NETWORK_INTERFACE),
             ('attachment.delete-on-termination', False),
             ('description', fakes.DESCRIPTION_NETWORK_INTERFACE_1),
             ('group-id', fakes.ID_EC2_SECURITY_GROUP_1),
             ('group-name', fakes.NAME_DEFAULT_OS_SECURITY_GROUP),
             ('mac-address', fakes.MAC_ADDRESS),
             ('network-interface-id', fakes.ID_EC2_NETWORK_INTERFACE_1),
             ('owner-id', fakes.ID_OS_PROJECT),
             ('private-ip-address', fakes.IP_NETWORK_INTERFACE_1),
             ('requester-managed', False),
             ('source-dest-check', True),
             ('status', 'available'),
             ('vpc-id', fakes.ID_EC2_VPC_1),
             ('subnet-id', fakes.ID_EC2_SUBNET_2)])
        self.check_tag_support(
            'DescribeNetworkInterfaces', 'networkInterfaceSet',
            fakes.ID_EC2_NETWORK_INTERFACE_1, 'networkInterfaceId')

    def test_describe_network_interface_attribute(self):
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_1)

        resp = self.execute(
            'DescribeNetworkInterfaceAttribute',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1,
             'Attribute': 'description'})
        self.assertEqual(fakes.ID_EC2_NETWORK_INTERFACE_1,
                         resp['networkInterfaceId'])
        self.assertEqual(fakes.DESCRIPTION_NETWORK_INTERFACE_1,
                         resp['description'].get('value', None))

    def test_modify_network_interface_attribute(self):
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_1,
                               fakes.DB_NETWORK_INTERFACE_2)

        self.execute(
            'ModifyNetworkInterfaceAttribute',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1,
             'Description.Value': 'New description'})
        self.db_api.update_item.assert_called_once_with(
            mock.ANY,
            tools.update_dict(fakes.DB_NETWORK_INTERFACE_1,
                              {'description': 'New description'}))

        self.db_api.reset_mock()
        self.execute(
            'ModifyNetworkInterfaceAttribute',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2,
             'Attachment.AttachmentId': (
                 fakes.ID_EC2_NETWORK_INTERFACE_2_ATTACH),
             'Attachment.DeleteOnTermination': 'True'})
        self.db_api.update_item.assert_called_once_with(
            mock.ANY,
            tools.update_dict(fakes.DB_NETWORK_INTERFACE_2,
                              {'delete_on_termination': True}))

    def test_modify_network_interface_attribute_invalid_parameters(self):
        self.assert_execution_error(
            'InvalidParameterCombination', 'ModifyNetworkInterfaceAttribute',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1,
             'Description.Value': 'New description',
             'SourceDestCheck.Value': 'True'})

        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_1,
                               fakes.DB_NETWORK_INTERFACE_2)

        self.assert_execution_error(
            'MissingParameter', 'ModifyNetworkInterfaceAttribute',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2,
             'Attachment.DeleteOnTermination': 'True'})

        self.assert_execution_error(
            'MissingParameter', 'ModifyNetworkInterfaceAttribute',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2,
             'Attachment.AttachmentId': (
                 fakes.ID_EC2_NETWORK_INTERFACE_2_ATTACH)})

        self.assert_execution_error(
            'InvalidAttachmentID.NotFound', 'ModifyNetworkInterfaceAttribute',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1,
             'Attachment.AttachmentId': (
                 fakes.ID_EC2_NETWORK_INTERFACE_2_ATTACH),
             'Attachment.DeleteOnTermination': 'True'})

        self.assert_execution_error(
            'InvalidAttachmentID.NotFound', 'ModifyNetworkInterfaceAttribute',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2,
             'Attachment.AttachmentId': ec2utils.change_ec2_id_kind(
                 fakes.ID_EC2_NETWORK_INTERFACE_1, 'eni-attach'),
             'Attachment.DeleteOnTermination': 'True'})

    def test_reset_network_interface_attribute(self):
        self.execute(
            'ResetNetworkInterfaceAttribute',
            {'NetworkInterfaceId':
             fakes.ID_EC2_NETWORK_INTERFACE_1,
             'Attribute': 'sourceDestCheck'})

    def test_attach_network_interface(self):
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_1,
                               fakes.DB_INSTANCE_1)
        self.neutron.show_port.return_value = (
            {'port': fakes.OS_PORT_1})
        self.isotime.return_value = fakes.TIME_ATTACH_NETWORK_INTERFACE
        self.execute(
            'AttachNetworkInterface',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1,
             'InstanceId': fakes.ID_EC2_INSTANCE_1,
             'DeviceIndex': '1'})
        self.nova.servers.interface_attach.assert_called_once_with(
            fakes.ID_OS_INSTANCE_1, fakes.ID_OS_PORT_1, None, None)
        self.db_api.update_item.assert_called_once_with(
            mock.ANY,
            tools.update_dict(
                fakes.DB_NETWORK_INTERFACE_1,
                {'device_index': 1,
                 'instance_id': fakes.ID_EC2_INSTANCE_1,
                 'delete_on_termination': False,
                 'attach_time': fakes.TIME_ATTACH_NETWORK_INTERFACE}))

    def test_attach_network_interface_invalid_parameters(self):
        # NOTE(ft): eni is already attached
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_2)
        self.assert_execution_error(
            'InvalidParameterValue', 'AttachNetworkInterface',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2,
             'InstanceId': fakes.ID_EC2_INSTANCE_2,
             'DeviceIndex': '1'})

        # NOTE(ft): device index is in use
        self.set_mock_db_items(
            fakes.DB_NETWORK_INTERFACE_1, fakes.DB_NETWORK_INTERFACE_2,
            fakes.DB_INSTANCE_1)
        self.assert_execution_error(
            'InvalidParameterValue', 'AttachNetworkInterface',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1,
             'InstanceId': fakes.ID_EC2_INSTANCE_1,
             'DeviceIndex': '0'})

    @tools.screen_unexpected_exception_logs
    def test_attach_network_interface_rollback(self):
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_1,
                               fakes.DB_INSTANCE_1)
        self.neutron.show_port.return_value = (
            {'port': fakes.OS_PORT_2})
        self.isotime.return_value = fakes.TIME_ATTACH_NETWORK_INTERFACE
        self.nova.servers.interface_attach.side_effect = Exception()

        self.assert_execution_error(
            self.ANY_EXECUTE_ERROR, 'AttachNetworkInterface',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1,
             'InstanceId': fakes.ID_EC2_INSTANCE_1,
             'DeviceIndex': '1'})

        self.db_api.update_item.assert_any_call(
            mock.ANY, fakes.DB_NETWORK_INTERFACE_1)

    def test_detach_network_interface(self):
        network_interface = tools.update_dict(fakes.DB_NETWORK_INTERFACE_2,
                                              {'device_index': 1})
        self.set_mock_db_items(network_interface)
        self.neutron.show_port.return_value = (
            {'port': fakes.OS_PORT_2})
        self.execute(
            'DetachNetworkInterface',
            {'AttachmentId': ec2utils.change_ec2_id_kind(
                    fakes.ID_EC2_NETWORK_INTERFACE_2, 'eni-attach')})
        self.neutron.update_port.assert_called_once_with(
            fakes.ID_OS_PORT_2,
            {'port': {'device_id': '',
                      'device_owner': ''}}
        )
        self.db_api.update_item.assert_called_once_with(
            mock.ANY,
            tools.purge_dict(fakes.DB_NETWORK_INTERFACE_2,
                             {'device_index',
                              'instance_id',
                              'delete_on_termination',
                              'attach_time'}))

    def test_detach_network_interface_invalid_parameters(self):
        # NOTE(ft): eni is not found
        self.set_mock_db_items()
        self.assert_execution_error(
            'InvalidAttachmentID.NotFound', 'DetachNetworkInterface',
            {'AttachmentId': ec2utils.change_ec2_id_kind(
                    fakes.ID_EC2_NETWORK_INTERFACE_2, 'eni-attach')})

        # NOTE(ft): eni is attached with device index = 0
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_2)
        self.assert_execution_error(
            'OperationNotPermitted', 'DetachNetworkInterface',
            {'AttachmentId': ec2utils.change_ec2_id_kind(
                    fakes.ID_EC2_NETWORK_INTERFACE_2, 'eni-attach')})

    @tools.screen_unexpected_exception_logs
    def test_detach_network_interface_rollback(self):
        network_interface = tools.update_dict(fakes.DB_NETWORK_INTERFACE_2,
                                              {'device_index': 1})
        self.set_mock_db_items(network_interface)
        self.neutron.show_port.return_value = (
            {'port': fakes.OS_PORT_2})
        self.neutron.update_port.side_effect = Exception()

        self.assert_execution_error(
            self.ANY_EXECUTE_ERROR, 'DetachNetworkInterface',
            {'AttachmentId': fakes.ID_EC2_NETWORK_INTERFACE_2_ATTACH})

        self.db_api.update_item.assert_any_call(
            mock.ANY, network_interface)

    def test_assign_unassign_private_ip_addresses(self):
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_1, fakes.DB_SUBNET_1)
        self.neutron.show_subnet.return_value = (
            {'subnet': fakes.OS_SUBNET_1})
        self.neutron.show_port.return_value = (
            {'port': copy.deepcopy(fakes.OS_PORT_1)})
        self.execute(
            'AssignPrivateIpAddresses',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1,
             'PrivateIpAddress.1': '10.10.1.5',
             'PrivateIpAddress.2': '10.10.1.6',
             })
        self.neutron.update_port.assert_called_once_with(
            fakes.ID_OS_PORT_1,
            {'port':
             {'fixed_ips': [
                 {'subnet_id': fakes.ID_OS_SUBNET_1,
                  'ip_address': fakes.IP_NETWORK_INTERFACE_1},
                 {'ip_address': '10.10.1.5'},
                 {'ip_address': '10.10.1.6'}]}})
        self.execute(
            'UnassignPrivateIpAddresses',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1,
             'PrivateIpAddress.1': '10.10.1.5',
             'PrivateIpAddress.2': '10.10.1.6',
             })
        self.neutron.update_port.assert_any_call(
            fakes.ID_OS_PORT_1,
            {'port':
             {'fixed_ips': [
                 {'subnet_id': fakes.ID_OS_SUBNET_1,
                  'ip_address': fakes.IP_NETWORK_INTERFACE_1}]}})

    def test_assign_private_ip_addresses_invalid_parameters(self):
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_1, fakes.DB_SUBNET_1)
        self.neutron.show_subnet.return_value = (
            {'subnet': fakes.OS_SUBNET_1})
        self.neutron.show_port.return_value = (
            {'port': copy.deepcopy(fakes.OS_PORT_1)})

        def do_check(error_code):
            self.assert_execution_error(
                error_code, 'AssignPrivateIpAddresses',
                {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_1,
                 'PrivateIpAddress.1': '10.10.1.5',
                 'PrivateIpAddress.2': '10.10.1.6',
                 })

        self.neutron.update_port.side_effect = (
            neutron_exception.IpAddressGenerationFailureClient())
        do_check('InsufficientFreeAddressesInSubnet')

        self.neutron.update_port.side_effect = (
            neutron_exception.IpAddressInUseClient())
        do_check('InvalidParameterValue')

        self.neutron.update_port.side_effect = (
            neutron_exception.BadRequest())
        do_check('InvalidParameterValue')

    def test_unassign_private_ip_addresses_invalid_parameters(self):
        self.set_mock_db_items(fakes.DB_NETWORK_INTERFACE_2, fakes.DB_SUBNET_2)
        self.neutron.show_subnet.return_value = (
            {'subnet': fakes.OS_SUBNET_2})
        self.neutron.show_port.return_value = (
            {'port': copy.deepcopy(fakes.OS_PORT_2)})

        self.assert_execution_error(
            'InvalidParameterValue', 'UnassignPrivateIpAddresses',
            {'NetworkInterfaceId': fakes.ID_EC2_NETWORK_INTERFACE_2,
             'PrivateIpAddress.1': '10.10.2.55'})



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
:mod:`ec2api` -- Cloud IaaS Platform
===================================

.. automodule:: ec2api
   :platform: Unix
   :synopsis: Infrastructure-as-a-Service Cloud platform.
"""

import six

import gettext


if six.PY2:
    gettext.install('ec2api', unicode=1)
else:
    gettext.install('ec2api')



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from oslo_config import cfg

from ec2api.tests.unit import base
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers
from ec2api.tests.unit import tools


class DhcpOptionsTestCase(base.ApiTestCase):

    def test_create_dhcp_options(self):

        def gen_opt(count, value):
            return 'DhcpConfiguration.' + str(count) + '.' + value

        def gen_ec2_param_dhcp_options(dhcp_options):
            dhcp_configuration = dhcp_options['dhcpConfigurationSet']
            result_param = {}
            opt_count = 0
            for opt in dhcp_configuration:
                opt_count += 1
                result_param[gen_opt(opt_count, 'Key')] = opt['key']
                value_count = 0
                for value in opt['valueSet']:
                    value_count += 1
                    result_param[gen_opt(opt_count,
                                         'Value.' + str(value_count))] = (
                                            str(value['value']))
            return result_param

        def check(ec2_fake, db_fake):
            self.db_api.add_item.return_value = db_fake
            resp = self.execute(
                    'CreateDhcpOptions',
                    gen_ec2_param_dhcp_options(ec2_fake))
            self.assertThat(ec2_fake, matchers.DictMatches(
                    resp['dhcpOptions'], orderless_lists=True))
            self.assert_any_call(self.db_api.add_item,
                                 mock.ANY, 'dopt',
                                 tools.purge_dict(db_fake, ('id',)))
            self.db_api.reset_mock()

        check(fakes.EC2_DHCP_OPTIONS_1, fakes.DB_DHCP_OPTIONS_1)
        check(fakes.EC2_DHCP_OPTIONS_2, fakes.DB_DHCP_OPTIONS_2)

    def test_create_dhcp_options_invalid_parameters(self):
        self.assert_execution_error(
            'InvalidParameterValue', 'CreateDhcpOptions',
            {'DhcpConfiguration.1.Key': 'InvalidParameter',
             'DhcpConfiguration.1.Value.1': 'Value'})

    def test_delete_dhcp_options(self):
        self.set_mock_db_items(fakes.DB_DHCP_OPTIONS_1)
        resp = self.execute('DeleteDhcpOptions',
                            {'dhcpOptionsId': fakes.ID_EC2_DHCP_OPTIONS_1})
        self.assertEqual(True, resp['return'])
        self.db_api.get_item_by_id.assert_any_call(
                mock.ANY,
                fakes.ID_EC2_DHCP_OPTIONS_1)
        self.db_api.get_items.assert_any_call(
                mock.ANY,
                'vpc')
        self.db_api.delete_item.assert_called_once_with(
                mock.ANY,
                fakes.ID_EC2_DHCP_OPTIONS_1)

    def test_delete_dhcp_options_with_dependencies(self):
        self.set_mock_db_items(
            fakes.DB_DHCP_OPTIONS_1,
            tools.update_dict(
                fakes.DB_VPC_1,
                {'dhcp_options_id': fakes.ID_EC2_DHCP_OPTIONS_1}))
        self.assert_execution_error(
            'DependencyViolation', 'DeleteDhcpOptions',
            {'dhcpOptionsId': fakes.ID_EC2_DHCP_OPTIONS_1})

    def test_describe_dhcp_options(self):
        self.set_mock_db_items(fakes.DB_DHCP_OPTIONS_1,
                               fakes.DB_DHCP_OPTIONS_2)

        resp = self.execute('DescribeDhcpOptions', {})
        self.assertThat(resp['dhcpOptionsSet'],
                        matchers.ListMatches([fakes.EC2_DHCP_OPTIONS_1,
                                              fakes.EC2_DHCP_OPTIONS_2],
                                             orderless_lists=True))

        resp = self.execute('DescribeDhcpOptions',
                            {'DhcpOptionsId.1': fakes.ID_EC2_DHCP_OPTIONS_1})
        self.assertThat(resp['dhcpOptionsSet'],
                        matchers.ListMatches([fakes.EC2_DHCP_OPTIONS_1],
                                             orderless_lists=True))
        self.db_api.get_items_by_ids.assert_called_once_with(
                mock.ANY, set([fakes.ID_EC2_DHCP_OPTIONS_1]))

        self.check_filtering(
            'DescribeDhcpOptions', 'dhcpOptionsSet',
            [('dhcp_options_id', fakes.ID_EC2_DHCP_OPTIONS_1),
             ('key', 'netbios-node-type'),
             ('value', '8.8.8.8')])
        self.check_tag_support(
            'DescribeDhcpOptions', 'dhcpOptionsSet',
            fakes.ID_EC2_DHCP_OPTIONS_1, 'dhcpOptionsId')

    def test_associate_dhcp_options(self):
        self.set_mock_db_items(fakes.DB_VPC_1, fakes.DB_DHCP_OPTIONS_1,
                               fakes.DB_NETWORK_INTERFACE_1)
        self.neutron.list_ports.return_value = (
                {'ports': [fakes.OS_PORT_1, fakes.OS_PORT_2]})

        def check(ec2_dhcp_options_id, db_dhcp_options_id, os_dhcp_options):
            resp = self.execute('AssociateDhcpOptions',
                                {'dhcpOptionsId': ec2_dhcp_options_id,
                                 'vpcId': fakes.ID_EC2_VPC_1})
            self.assertEqual(True, resp['return'])
            self.db_api.update_item.assert_any_call(
                    mock.ANY,
                    tools.update_dict(
                            fakes.DB_VPC_1,
                            {'dhcp_options_id': db_dhcp_options_id}))
            self.assert_any_call(
                self.neutron.update_port,
                fakes.ID_OS_PORT_1,
                {'port': self._effective_os_dhcp_options(os_dhcp_options)})

        check(fakes.ID_EC2_DHCP_OPTIONS_1, fakes.ID_EC2_DHCP_OPTIONS_1,
              fakes.OS_DHCP_OPTIONS_1)

        check('default', None, {'extra_dhcp_opts': []})

    @tools.screen_unexpected_exception_logs
    def test_associate_dhcp_options_rollback(self):
        vpc = tools.update_dict(
                fakes.DB_VPC_1,
                {'dhcp_options_id': fakes.ID_EC2_DHCP_OPTIONS_1})
        self.set_mock_db_items(
            vpc, fakes.DB_DHCP_OPTIONS_1, fakes.DB_DHCP_OPTIONS_2,
            fakes.DB_NETWORK_INTERFACE_1, fakes.DB_NETWORK_INTERFACE_2)
        self.neutron.list_ports.return_value = (
                {'ports': [fakes.OS_PORT_1, fakes.OS_PORT_2]})

        def update_port_func(port_id, _port_data):
            if port_id == fakes.ID_OS_PORT_2:
                raise Exception()

        self.neutron.update_port.side_effect = update_port_func

        self.assert_execution_error(
            self.ANY_EXECUTE_ERROR, 'AssociateDhcpOptions',
            {'dhcpOptionsId': fakes.ID_EC2_DHCP_OPTIONS_2,
             'vpcId': fakes.ID_EC2_VPC_1})

        self.assert_any_call(self.neutron.update_port,
                             fakes.ID_OS_PORT_1,
                             {'port': fakes.OS_DHCP_OPTIONS_1})
        self.db_api.update_item.assert_any_call(
                mock.ANY, vpc)

    def _effective_os_dhcp_options(self, os_dhcp_options):
        CONF = cfg.CONF
        dhcp_opts = {
            'extra_dhcp_opts': [{'opt_name': 'mtu',
                                 'opt_value': str(CONF.network_device_mtu)}]}
        dhcp_opts['extra_dhcp_opts'].extend(
            os_dhcp_options.get('extra_dhcp_opts', []))
        return dhcp_opts



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from ec2api.api import image as image_api
from ec2api.api import instance as instance_api
from ec2api.api import snapshot as snapshot_api
from ec2api.api import volume as volume_api
from ec2api.db import api as db_api
from ec2api import exception
from ec2api.tests.unit import base
from ec2api.tests.unit import fakes


class DBItemsAutoCreationTestCase(base.DbTestCase):

    def setUp(self):
        super(DBItemsAutoCreationTestCase, self).setUp()
        self.mock_all_os()
        self.context = base.create_context()

    def assert_image_project(self, expected_project_id, image_id):
        if expected_project_id:
            context = mock.NonCallableMock(project_id=expected_project_id)
        else:
            context = self.context
        image_item = db_api.get_item_by_id(context, image_id)
        if expected_project_id:
            self.assertIsNotNone(image_item)
        else:
            self.assertIsNone(image_item)

    def test_describe_new_instance_then_its_volume(self):
        os_instance_id = fakes.random_os_id()
        os_volume_id = fakes.random_os_id()
        os_instance = {
            'id': os_instance_id,
            'flavor': {'id': 'fake'},
            'volumes_attached': [{'id': os_volume_id}],
        }
        os_volume = {
            'id': os_volume_id,
            'status': 'in-use',
            'attachments': [{'device': '/dev/vdb',
                             'server_id': os_instance_id}],
        }
        self.nova_admin.servers.list.return_value = [
            fakes.OSInstance_full(os_instance)]
        self.cinder.volumes.list.return_value = [
            fakes.OSVolume(os_volume)]

        reservations = instance_api.describe_instances(self.context)
        instance = reservations['reservationSet'][0]['instancesSet'][0]
        volume_id = instance['blockDeviceMapping'][0]['ebs']['volumeId']
        volume_api.describe_volumes(self.context, [volume_id])

    def _test_describe_new_images(self, image_project_id=None,
                                  aki_image_project_id=None,
                                  with_id_mapping=False):
        os_image_id = fakes.random_os_id()
        os_aki_image_id = fakes.random_os_id()
        os_image = {
            'id': os_image_id,
            'owner': image_project_id,
            'is_public': True,
            'container_format': 'ami',
            'kernel_id': os_aki_image_id,
        }
        os_aki_image = {
            'id': os_aki_image_id,
            'owner': aki_image_project_id,
            'is_public': True,
            'container_format': 'aki',
        }
        self.glance.images.list.return_value = (
            [fakes.OSImage(os_image), fakes.OSImage(os_aki_image)]
            if with_id_mapping else
            [fakes.OSImage(os_aki_image), fakes.OSImage(os_image)])

        images = image_api.describe_images(self.context)
        image = next(i for i in images['imagesSet']
                     if i['imageType'] == 'machine')
        aki_image = next(i for i in images['imagesSet']
                         if i['imageType'] == 'kernel')
        self.assertEqual(image_project_id, image['imageOwnerId'])
        self.assert_image_project(
            (image_project_id
             if image_project_id == fakes.ID_OS_PROJECT else
             None),
            image['imageId'])
        self.assertEqual(aki_image_project_id, aki_image['imageOwnerId'])
        self.assert_image_project(
            (aki_image_project_id
             if aki_image_project_id == fakes.ID_OS_PROJECT else
             None),
            aki_image['imageId'])

    def test_describe_new_alien_images(self):
        alien_project_id = fakes.random_os_id()
        self._test_describe_new_images(
            image_project_id=alien_project_id,
            aki_image_project_id=alien_project_id,
            with_id_mapping=False)

    def test_describe_new_local_images(self):
        self._test_describe_new_images(
            image_project_id=fakes.ID_OS_PROJECT,
            aki_image_project_id=fakes.ID_OS_PROJECT,
            with_id_mapping=False)

    def test_describe_new_local_ami_alien_aki_images(self):
        alien_project_id = fakes.random_os_id()
        self._test_describe_new_images(
            image_project_id=fakes.ID_OS_PROJECT,
            aki_image_project_id=alien_project_id,
            with_id_mapping=False)

    def test_describe_new_alien_ami_local_aki_images(self):
        alien_project_id = fakes.random_os_id()
        self._test_describe_new_images(
            image_project_id=alien_project_id,
            aki_image_project_id=fakes.ID_OS_PROJECT,
            with_id_mapping=False)

    def test_describe_new_alien_images_with_mappings(self):
        alien_project_id = fakes.random_os_id()
        self._test_describe_new_images(
            image_project_id=alien_project_id,
            aki_image_project_id=alien_project_id,
            with_id_mapping=True)

    def test_describe_new_local_images_with_mappings(self):
        self._test_describe_new_images(
            image_project_id=fakes.ID_OS_PROJECT,
            aki_image_project_id=fakes.ID_OS_PROJECT,
            with_id_mapping=True)

    def test_describe_new_local_ami_alien_aki_images_with_mappings(self):
        alien_project_id = fakes.random_os_id()
        self._test_describe_new_images(
            image_project_id=fakes.ID_OS_PROJECT,
            aki_image_project_id=alien_project_id,
            with_id_mapping=True)

    def test_describe_new_alien_ami_local_aki_images_with_mappings(self):
        alien_project_id = fakes.random_os_id()
        self._test_describe_new_images(
            image_project_id=alien_project_id,
            aki_image_project_id=fakes.ID_OS_PROJECT,
            with_id_mapping=True)

    def _get_new_ebs_image(self, image_project_id=None,
                           bdm_image_project_id=None):
        os_image_id = fakes.random_os_id()
        os_snapshot_id = fakes.random_os_id()
        os_bdm_image_id = fakes.random_os_id()
        os_image = {
            'id': os_image_id,
            'owner': image_project_id,
            'is_public': True,
            'container_format': 'ami',
            'bdm_v2': True,
            'block_device_mapping': [{'device_name': '/dev/vds',
                                          'source_type': 'snapshot',
                                          'destination_type': 'volume',
                                          'snapshot_id': os_snapshot_id}],
        }
        if os_bdm_image_id:
            os_image['block_device_mapping'].append({
                'device_name': '/dev/vdi',
                'source_type': 'image',
                'destination_type': 'volume',
                'image_id': os_bdm_image_id,
                'size': 100})
        os_snapshot = {
            'id': os_snapshot_id,
        }
        os_bdm_image = {
            'id': os_bdm_image_id,
            'owner': bdm_image_project_id,
            'is_public': True,
        }
        os_images = [fakes.OSImage(os_image)]
        if bdm_image_project_id:
            os_images.append(fakes.OSImage(os_bdm_image))
        self.glance.images.list.return_value = os_images
        self.cinder.volume_snapshots.list.return_value = (
            [fakes.OSSnapshot(os_snapshot)]
            if image_project_id == fakes.ID_OS_PROJECT else
            [])

        images = image_api.describe_images(self.context)
        return next(i for i in images['imagesSet']
                    if i['blockDeviceMapping'])

    def _find_snapshot_id_in_bdm(self, image, device_name):
        return next(bdm['ebs']['snapshotId']
                    for bdm in image['blockDeviceMapping']
                    if bdm['deviceName'] == device_name)

    def test_describe_new_local_snapshot_from_new_image(self):
        image = self._get_new_ebs_image(image_project_id=fakes.ID_OS_PROJECT)
        snapshot_id = self._find_snapshot_id_in_bdm(image, '/dev/vds')
        snapshot_api.describe_snapshots(self.context, [snapshot_id])

    def test_describe_new_alien_snapshot_from_new_image(self):
        image = self._get_new_ebs_image(image_project_id=fakes.random_os_id())
        snapshot_id = self._find_snapshot_id_in_bdm(image, '/dev/vds')
        self.assertRaises(exception.InvalidSnapshotNotFound,
                          snapshot_api.describe_snapshots,
                          self.context, [snapshot_id])

    def test_describe_new_local_bdm_image_from_local_image(self):
        image = self._get_new_ebs_image(
            image_project_id=fakes.ID_OS_PROJECT,
            bdm_image_project_id=fakes.ID_OS_PROJECT)
        image_id = self._find_snapshot_id_in_bdm(image, '/dev/vdi')
        image_api.describe_images(self.context, image_id=[image_id])
        self.assert_image_project(fakes.ID_OS_PROJECT, image_id)

    def test_describe_new_alien_bdm_image_from_new_local_image(self):
        alien_project_id = fakes.random_os_id()
        image = self._get_new_ebs_image(
            image_project_id=fakes.ID_OS_PROJECT,
            bdm_image_project_id=alien_project_id)
        image_id = self._find_snapshot_id_in_bdm(image, '/dev/vdi')
        image_api.describe_images(self.context, image_id=[image_id])
        self.assert_image_project(None, image_id)

    def test_describe_new_alien_bdm_image_from_new_alien_image(self):
        alien_project_id = fakes.random_os_id()
        image = self._get_new_ebs_image(
            image_project_id=alien_project_id,
            bdm_image_project_id=alien_project_id)
        image_id = self._find_snapshot_id_in_bdm(image, '/dev/vdi')
        image_api.describe_images(self.context, image_id=[image_id])
        self.assert_image_project(None, image_id)

    def _test_describe_new_instance_then_its_image(self, image_project_id):
        os_instance_id = fakes.random_os_id()
        os_image_id = fakes.random_os_id()
        os_instance = {
            'id': os_instance_id,
            'flavor': {'id': 'fake'},
            'image': {'id': os_image_id},
        }
        os_image = {
            'id': os_image_id,
            'owner': image_project_id,
            'visibility': 'public',
        }
        self.nova_admin.servers.list.return_value = [
            fakes.OSInstance_full(os_instance)]
        self.glance.images.list.return_value = [fakes.OSImage(os_image)]

        reservations = instance_api.describe_instances(self.context)
        instance = reservations['reservationSet'][0]['instancesSet'][0]
        image_id = instance['imageId']
        image = (image_api.describe_images(self.context, image_id=[image_id])
                 ['imagesSet'][0])
        self.assertEqual(image_id, image['imageId'])
        self.assertEqual(image_project_id, image['imageOwnerId'])
        expected_project_id = (fakes.ID_OS_PROJECT
                               if image_project_id == fakes.ID_OS_PROJECT else
                               None)
        self.assert_image_project(expected_project_id, image['imageId'])

    def test_describe_new_instance_then_its_local_image(self):
        self._test_describe_new_instance_then_its_image(fakes.ID_OS_PROJECT)

    def test_describe_new_instance_then_its_alien_image(self):
        self._test_describe_new_instance_then_its_image(fakes.random_os_id())

    def test_describe_new_instance_then_its_alien_image_attribute(self):
        os_instance_id = fakes.random_os_id()
        os_image_id = fakes.random_os_id()
        alien_project_id = fakes.random_os_id()
        os_instance = {
            'id': os_instance_id,
            'flavor': {'id': 'fake'},
            'image': {'id': os_image_id},
        }
        os_image = {
            'id': os_image_id,
            'owner': alien_project_id,
            'is_public': True,
        }
        self.nova_admin.servers.list.return_value = [
            fakes.OSInstance_full(os_instance)]
        self.glance.images.get.return_value = fakes.OSImage(os_image)

        reservations = instance_api.describe_instances(self.context)
        instance = reservations['reservationSet'][0]['instancesSet'][0]
        image_id = instance['imageId']

        # NOTE(ft): ensure that InvalidAMIID.NotFound is not raised
        self.assertRaises(exception.AuthFailure,
                          image_api.describe_image_attribute,
                          self.context, image_id, 'description')



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
import copy
from unittest import mock

from novaclient import exceptions as nova_exception
from oslo_cache import core as cache_core
from oslo_config import cfg
import six

from ec2api import exception
from ec2api.metadata import api
from ec2api.tests.unit import base
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers
from ec2api.tests.unit import tools

CONF = cfg.CONF
FAKE_USER_DATA = u'fake_user_data-' + six.unichr(1071)


class MetadataApiTestCase(base.ApiTestCase):
    # TODO(ft): 'execute' feature isn't used here, but some mocks and
    # fake context are. ApiTestCase should be split to some classes to use
    # its feature optimally

    def setUp(self):
        super(MetadataApiTestCase, self).setUp()
        self.instance_api = self.mock('ec2api.metadata.api.instance_api')

        self.set_mock_db_items(fakes.DB_INSTANCE_1)
        self.instance_api.describe_instances.return_value = {
               'reservationSet': [fakes.EC2_RESERVATION_1]}
        userDataValue = base64.b64encode(FAKE_USER_DATA.encode('utf-8'))
        self.instance_api.describe_instance_attribute.return_value = {
                'instanceId': fakes.ID_EC2_INSTANCE_1,
                'userData': {'value': userDataValue}}
        self.configure(enabled=False, group='cache')
        self._init_cache_region()

        self.fake_context = base.create_context()

    def _init_cache_region(self):
        self.cache_region = cache_core.create_region()
        cache_core.configure_cache_region(CONF, self.cache_region)

    def test_get_version_list(self):
        retval = api.get_version_list()
        self.assertEqual('\n'.join(api.VERSIONS + ['latest']), retval)

    def test_get_instance_and_project_id_by_provider_id(self):
        self.neutron.list_subnets.return_value = {
            'subnets': [fakes.OS_SUBNET_1, fakes.OS_SUBNET_2]}
        self.neutron.list_ports.return_value = {
            'ports': [fakes.OS_PORT_2]}
        self.assertEqual(
            (fakes.ID_OS_INSTANCE_1, fakes.ID_OS_PROJECT),
            api.get_os_instance_and_project_id_by_provider_id(
                self.fake_context, mock.sentinel.provider_id,
                fakes.IP_NETWORK_INTERFACE_2))
        self.neutron.list_subnets.assert_called_with(
            advanced_service_providers=[mock.sentinel.provider_id],
            fields=['network_id'])
        self.neutron.list_ports.assert_called_with(
            fixed_ips=('ip_address=%s' % fakes.IP_NETWORK_INTERFACE_2),
            network_id=[fakes.ID_OS_NETWORK_1, fakes.ID_OS_NETWORK_2],
            fields=['device_id', 'tenant_id'])

        self.neutron.list_ports.return_value = {'ports': []}
        self.assertRaises(exception.EC2MetadataNotFound,
                          api.get_os_instance_and_project_id_by_provider_id,
                          self.fake_context, mock.sentinel.provider_id,
                          fakes.IP_NETWORK_INTERFACE_2)

        self.neutron.list_subnets.return_value = {'subnets': []}
        self.assertRaises(exception.EC2MetadataNotFound,
                          api.get_os_instance_and_project_id_by_provider_id,
                          self.fake_context, mock.sentinel.provider_id,
                          fakes.IP_NETWORK_INTERFACE_2)

    def test_get_version_root(self):
        retval = api.get_metadata_item(self.fake_context, ['2009-04-04'],
                                       fakes.ID_OS_INSTANCE_1,
                                       fakes.IP_NETWORK_INTERFACE_2,
                                       self.cache_region)
        self.assertEqual('meta-data/\nuser-data', retval)

        self.assertRaises(
              exception.EC2MetadataNotFound,
              api.get_metadata_item, self.fake_context, ['9999-99-99'],
              fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
              self.cache_region)

        self.db_api.get_items_ids.assert_called_with(
            self.fake_context, 'i', item_ids=None,
            item_os_ids=(fakes.ID_OS_INSTANCE_1,))
        self.instance_api.describe_instances.assert_called_with(
            self.fake_context, [fakes.ID_EC2_INSTANCE_1])
        self.instance_api.describe_instance_attribute.assert_called_with(
            self.fake_context, fakes.ID_EC2_INSTANCE_1, 'userData')

    def test_invalid_path(self):
        self.assertRaises(exception.EC2MetadataNotFound,
                          api.get_metadata_item, self.fake_context,
                          ['9999-99-99', 'user-data-invalid'],
                          fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
                          self.cache_region)

    def test_mismatch_project_id(self):
        self.fake_context.project_id = fakes.random_os_id()
        self.assertRaises(
              exception.EC2MetadataNotFound,
              api.get_metadata_item, self.fake_context, ['2009-04-04'],
              fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
              self.cache_region)

    def test_non_existing_instance(self):
        self.instance_api.describe_instances.return_value = {
               'reservationSet': []}
        self.assertRaises(
              exception.EC2MetadataNotFound,
              api.get_metadata_item, self.fake_context, ['2009-04-04'],
              fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
              self.cache_region)

    def test_user_data(self):
        retval = api.get_metadata_item(
               self.fake_context, ['2009-04-04', 'user-data'],
               fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
               self.cache_region)
        self.assertEqual(FAKE_USER_DATA, retval)

    def test_no_user_data(self):
        self.instance_api.describe_instance_attribute.return_value = {
                'instanceId': fakes.ID_EC2_INSTANCE_1}
        self.assertRaises(
              exception.EC2MetadataNotFound,
              api.get_metadata_item, self.fake_context,
              ['2009-04-04', 'user-data'],
              fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
              self.cache_region)

    def test_security_groups(self):
        self.instance_api.describe_instances.return_value = {
               'reservationSet': [fakes.EC2_RESERVATION_2]}
        retval = api.get_metadata_item(
               self.fake_context,
               ['2009-04-04', 'meta-data', 'security-groups'],
               fakes.ID_OS_INSTANCE_2, fakes.IP_NETWORK_INTERFACE_1,
               self.cache_region)
        self.assertEqual('\n'.join(['groupname3']),
                         retval)

    def test_local_hostname(self):
        retval = api.get_metadata_item(
               self.fake_context,
               ['2009-04-04', 'meta-data', 'local-hostname'],
               fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
               self.cache_region)
        self.assertEqual(fakes.EC2_INSTANCE_1['privateDnsName'], retval)

    def test_local_ipv4(self):
        retval = api.get_metadata_item(
               self.fake_context,
               ['2009-04-04', 'meta-data', 'local-ipv4'],
               fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
               self.cache_region)
        self.assertEqual(fakes.IP_NETWORK_INTERFACE_2, retval)

    def test_local_ipv4_from_address(self):
        self.instance_api.describe_instances.return_value = {
               'reservationSet': [fakes.EC2_RESERVATION_2]}
        retval = api.get_metadata_item(
               self.fake_context,
               ['2009-04-04', 'meta-data', 'local-ipv4'],
               fakes.ID_OS_INSTANCE_2, fakes.IP_NETWORK_INTERFACE_1,
               self.cache_region)
        self.assertEqual(fakes.IP_NETWORK_INTERFACE_1, retval)

    def test_pubkey_name(self):
        retval = api.get_metadata_item(
               self.fake_context,
               ['2009-04-04', 'meta-data', 'public-keys'],
               fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
               self.cache_region)
        self.assertEqual('0=%s' % fakes.NAME_KEY_PAIR, retval)

    def test_pubkey(self):
        self.nova.servers.get.return_value = (
               fakes.OSInstance(fakes.OS_INSTANCE_1))
        self.nova.keypairs.keypair_prefix = 'os_keypairs'
        self.nova.keypairs._get.return_value = (
               fakes.NovaKeyPair(fakes.OS_KEY_PAIR))
        retval = api.get_metadata_item(
               self.fake_context,
               ['2009-04-04', 'meta-data', 'public-keys', '0', 'openssh-key'],
               fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
               self.cache_region)
        self.assertEqual(fakes.PUBLIC_KEY_KEY_PAIR, retval)
        self.nova.servers.get.assert_called_once_with(fakes.ID_OS_INSTANCE_1)
        self.nova.keypairs._get.assert_called_once_with(
               '/os_keypairs/%s?user_id=%s' % (fakes.NAME_KEY_PAIR,
                                               fakes.ID_OS_USER),
               'keypair')

        self.nova.keypairs._get.side_effect = nova_exception.NotFound(404)
        self.assertRaises(
                exception.EC2MetadataNotFound,
                api.get_metadata_item,
                self.fake_context,
                ['2009-04-04', 'meta-data', 'public-keys', '0', 'openssh-key'],
                fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
                self.cache_region)

    def test_image_type_ramdisk(self):
        retval = api.get_metadata_item(
               self.fake_context,
               ['2009-04-04', 'meta-data', 'ramdisk-id'],
               fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
               self.cache_region)
        self.assertEqual(fakes.ID_EC2_IMAGE_ARI_1, retval)

    def test_image_type_kernel(self):
        retval = api.get_metadata_item(
               self.fake_context,
               ['2009-04-04', 'meta-data', 'kernel-id'],
               fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
               self.cache_region)
        self.assertEqual(fakes.ID_EC2_IMAGE_AKI_1, retval)

    def test_check_version(self):
        retval = api.get_metadata_item(
               self.fake_context,
               ['2009-04-04', 'meta-data', 'block-device-mapping'],
               fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
               self.cache_region)
        self.assertIsNotNone(retval)

        self.assertRaises(
              exception.EC2MetadataNotFound,
              api.get_metadata_item, self.fake_context,
              ['2007-08-29', 'meta-data', 'block-device-mapping'],
              fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
              self.cache_region)

    def test_format_instance_mapping(self):
        retval = api._build_block_device_mappings(
                'fake_context', fakes.EC2_INSTANCE_1, fakes.ID_OS_INSTANCE_1)
        self.assertThat(retval,
                        matchers.DictMatches(
                             {'ami': 'vda',
                              'root': fakes.ROOT_DEVICE_NAME_INSTANCE_1}))

        retval = api._build_block_device_mappings(
                'fake_context', fakes.EC2_INSTANCE_2, fakes.ID_OS_INSTANCE_2)
        expected = {'ami': 'sdb1',
                    'root': fakes.ROOT_DEVICE_NAME_INSTANCE_2}
        expected.update(fakes.EC2_BDM_METADATA_INSTANCE_2)
        self.assertThat(retval,
                        matchers.DictMatches(expected))

    def test_metadata_cache(self):
        self.configure(enabled=True, group='cache')
        self.configure(backend='oslo_cache.dict', group='cache')
        self._init_cache_region()
        retval = api.get_metadata_item(
               self.fake_context,
               ['2009-04-04', 'meta-data', 'local-ipv4'],
               fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
               self.cache_region)
        self.assertEqual(fakes.IP_NETWORK_INTERFACE_2, retval)
        self.nova.servers.get.assert_called_once_with(fakes.ID_OS_INSTANCE_1)
        self.nova.servers.get.reset_mock()

        retval = api.get_metadata_item(
               self.fake_context,
               ['2009-04-04', 'meta-data', 'instance-id'],
               fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
               self.cache_region)
        self.assertEqual(fakes.ID_EC2_INSTANCE_1, retval)
        self.nova.servers.get.assert_not_called()


class MetadataApiIntegralTestCase(base.ApiTestCase):
    # TODO(ft): 'execute' feature isn't used here, but some mocks and
    # fake context are. ApiTestCase should be split to some classes to use
    # its feature optimally

    @mock.patch('ec2api.metadata.api.cache_core.create_region')
    @mock.patch('ec2api.api.instance.security_group_api')
    @mock.patch('ec2api.api.instance.network_interface_api')
    def test_get_metadata_integral(self, network_interface_api,
                                   security_group_api, create_region):
        fake_context = base.create_context(is_os_admin=True)

        self.set_mock_db_items(
            fakes.DB_INSTANCE_1, fakes.DB_INSTANCE_2,
            fakes.DB_NETWORK_INTERFACE_1, fakes.DB_NETWORK_INTERFACE_2,
            fakes.DB_IMAGE_1, fakes.DB_IMAGE_2,
            fakes.DB_IMAGE_ARI_1, fakes.DB_IMAGE_AKI_1,
            fakes.DB_VOLUME_1, fakes.DB_VOLUME_2, fakes.DB_VOLUME_3)
        self.nova_admin.servers.list.return_value = [
            fakes.OSInstance_full(fakes.OS_INSTANCE_1),
            fakes.OSInstance_full(fakes.OS_INSTANCE_2)]
        self.nova_admin.servers.get.side_effect = tools.get_by_1st_arg_getter({
            fakes.ID_OS_INSTANCE_1: fakes.OSInstance_full(fakes.OS_INSTANCE_1),
            fakes.ID_OS_INSTANCE_2: fakes.OSInstance_full(fakes.OS_INSTANCE_2)
        })
        self.nova_admin.keypairs._get.return_value = (
               fakes.NovaKeyPair(fakes.OS_KEY_PAIR))
        self.cinder.volumes.list.return_value = [
            fakes.OSVolume(fakes.OS_VOLUME_1),
            fakes.OSVolume(fakes.OS_VOLUME_2),
            fakes.OSVolume(fakes.OS_VOLUME_3)]
        network_interface_api.describe_network_interfaces.side_effect = (
            lambda *args, **kwargs: copy.deepcopy({
                'networkInterfaceSet': [fakes.EC2_NETWORK_INTERFACE_1,
                                        fakes.EC2_NETWORK_INTERFACE_2]}))
        security_group_api.describe_security_groups.return_value = {
            'securityGroupInfo': [fakes.EC2_SECURITY_GROUP_1,
                                  fakes.EC2_SECURITY_GROUP_3]}
        create_region.get.return_value = cache_core.NO_VALUE

        retval = api.get_metadata_item(
               fake_context, ['latest', 'meta-data', 'instance-id'],
               fakes.ID_OS_INSTANCE_1, fakes.IP_NETWORK_INTERFACE_2,
               create_region)
        self.assertEqual(fakes.ID_EC2_INSTANCE_1, retval)

        retval = api.get_metadata_item(
               fake_context, ['latest', 'meta-data', 'instance-id'],
               fakes.ID_OS_INSTANCE_2, '10.200.1.15',
               create_region)
        self.assertEqual(fakes.ID_EC2_INSTANCE_2, retval)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from ec2api.api import ec2utils
from ec2api.api import tag as tag_api
from ec2api.tests.unit import base
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers


class TagTestCase(base.ApiTestCase):

    def test_create_tags(self):
        self.db_api.get_item_by_id.return_value = {'id': 'fake'}

        # NOTE(ft): check create several tags for several resources
        resp = self.execute('CreateTags',
                            {'ResourceId.1': fakes.ID_EC2_VPC_1,
                             'ResourceId.2': fakes.ID_EC2_SUBNET_1,
                             'Tag.1.Key': 'private',
                             'Tag.1.Value': '',
                             'Tag.2.Key': 'admin',
                             'Tag.2.Value': 'John Smith'})
        self.assertEqual({'return': True}, resp)
        self.assertEqual(1, self.db_api.add_tags.call_count)
        self.assertEqual(2, len(self.db_api.add_tags.call_args))
        self.assertThat(self.db_api.add_tags.call_args[0][1],
                        matchers.ListMatches(
                             [{'item_id': fakes.ID_EC2_VPC_1,
                               'key': 'private',
                               'value': ''},
                              {'item_id': fakes.ID_EC2_SUBNET_1,
                               'key': 'private',
                               'value': ''},
                              {'item_id': fakes.ID_EC2_VPC_1,
                               'key': 'admin',
                               'value': 'John Smith'},
                              {'item_id': fakes.ID_EC2_SUBNET_1,
                               'key': 'admin',
                               'value': 'John Smith'}],
                             orderless_lists=True))

        # NOTE(ft): check a tag can be created for all valid resource types
        resource_ids = [fakes.random_ec2_id(r_t)
                        for r_t in ['dopt', 'ami', 'aki', 'ari', 'cgw', 'i',
                                    'igw', 'eni', 'rtb', 'snap', 'subnet',
                                    'sg', 'vgw', 'vol', 'vpc', 'vpn']]
        self.assertEqual(len(resource_ids), len(tag_api.RESOURCE_TYPES))

        params = {'ResourceId.%s' % num: r_id
                  for num, r_id in enumerate(resource_ids)}
        params.update({'Tag.1.Key': 'tag',
                       'Tag.1.Value': 'value'})
        resp = self.execute('CreateTags', params)
        self.assertEqual({'return': True}, resp)

        # NOTE(ft): check create a tag for non-existing images
        self.db_api.get_item_by_id.return_value = None
        resp = self.execute('CreateTags',
                            {'ResourceId.1': fakes.ID_EC2_IMAGE_1,
                             'ResourceId.2': fakes.ID_EC2_IMAGE_AKI_1,
                             'ResourceId.3': fakes.ID_EC2_IMAGE_ARI_1,
                             'Tag.1.Key': 'Oracle RAC node',
                             'Tag.1.Value': ''})
        self.assertEqual({'return': True}, resp)

    def test_create_tags_invalid_parameters(self):
        # NOTE(ft): check tag validity checks
        self.assert_execution_error('InvalidParameterValue', 'CreateTags',
                                    {'ResourceId.1': fakes.ID_EC2_VPC_1,
                                     'Tag.1.Value': ''})

        self.assert_execution_error('InvalidParameterValue', 'CreateTags',
                                    {'ResourceId.1': fakes.ID_EC2_VPC_1,
                                     'Tag.1.Key': ''})

        self.assert_execution_error('InvalidParameterValue', 'CreateTags',
                                    {'ResourceId.1': fakes.ID_EC2_VPC_1,
                                     'Tag.1.Key': 'a' * 128})

        self.assert_execution_error('InvalidParameterValue', 'CreateTags',
                                    {'ResourceId.1': fakes.ID_EC2_VPC_1,
                                     'Tag.1.Key': 'fake-key',
                                     'Tag.1.Value': 'a' * 256})

        # NOTE(ft): check resource type check
        self.assert_execution_error(
            'InvalidID', 'CreateTags',
            {'ResourceId.1': fakes.random_ec2_id('fake'),
             'Tag.1.Key': 'fake-key',
             'Tag.1.Value': 'fake-value'})

        # NOTE(ft): check resource existence check
        self.db_api.get_item_by_id.return_value = None
        for r_id in tag_api.RESOURCE_TYPES:
            if r_id in ('ami', 'ari', 'aki'):
                continue
            exc_class = ec2utils.NOT_FOUND_EXCEPTION_MAP[r_id]
            try:
                error_code = exc_class.ec2_code
            except AttributeError:
                error_code = exc_class.__name__
            self.assert_execution_error(
                error_code, 'CreateTags',
                {'ResourceId.1': fakes.random_ec2_id(r_id),
                 'Tag.1.Key': 'fake-key',
                 'Tag.1.Value': 'fake-value'})

    def test_delete_tag(self):
        resp = self.execute('DeleteTags',
                            {'ResourceId.1': fakes.ID_EC2_VPC_1,
                             'ResourceId.2': fakes.ID_EC2_SUBNET_1,
                             'Tag.1.Key': 'key1',
                             'Tag.2.Value': 'value2',
                             'Tag.3.Key': 'key3',
                             'Tag.3.Value': 'value3'})
        self.assertEqual({'return': True}, resp)
        self.db_api.delete_tags.assert_called_once_with(
            mock.ANY, [fakes.ID_EC2_VPC_1, fakes.ID_EC2_SUBNET_1],
            [{'key': 'key1'},
             {'value': 'value2'},
             {'key': 'key3',
              'value': 'value3'}])

        resp = self.execute('DeleteTags',
                            {'ResourceId.1': fakes.ID_EC2_VPC_1})
        self.assertEqual({'return': True}, resp)
        self.db_api.delete_tags.assert_called_with(
            mock.ANY, [fakes.ID_EC2_VPC_1], None)

    def test_describe_tags(self):
        self.db_api.get_tags.return_value = [{'item_id': fakes.ID_EC2_VPC_1,
                                              'key': 'key1',
                                              'value': ''},
                                             {'item_id': fakes.ID_EC2_VPC_2,
                                              'key': 'key2',
                                              'value': 'value2'},
                                             {'item_id': fakes.ID_EC2_VPC_2,
                                              'key': 'key1',
                                              'value': 'value3'}
                                             ]
        resp = self.execute('DescribeTags', {})
        self.assertThat(resp,
                        matchers.DictMatches(
                            {'tagSet': [{'resourceType': 'vpc',
                                         'resourceId': fakes.ID_EC2_VPC_1,
                                         'key': 'key1',
                                         'value': None},
                                        {'resourceType': 'vpc',
                                         'resourceId': fakes.ID_EC2_VPC_2,
                                         'key': 'key2',
                                         'value': 'value2'},
                                        {'resourceType': 'vpc',
                                         'resourceId': fakes.ID_EC2_VPC_2,
                                         'key': 'key1',
                                         'value': 'value3'}
                                        ]},
                            orderless_lists=True),
                        verbose=True)

        self.check_filtering(
            'DescribeTags', 'tagSet',
            [('resource-type', 'vpc'),
             ('resource-id', fakes.ID_EC2_VPC_1),
             ('key', 'key1'),
             ('value', 'value2')])

        # NOTE(ft): check all resource types are displayed correctly
        for r_id, r_type in [('dopt', 'dhcp-options'),
                             ('ami', 'image'),
                             ('aki', 'image'),
                             ('ari', 'image'),
                             ('cgw', 'customer-gateway'),
                             ('i', 'instance'),
                             ('igw', 'internet-gateway'),
                             ('eni', 'network-interface'),
                             ('rtb', 'route-table'),
                             ('snap', 'snapshot'),
                             ('subnet', 'subnet'),
                             ('sg', 'security-group'),
                             ('vgw', 'vpn-gateway'),
                             ('vol', 'volume'),
                             ('vpc', 'vpc'),
                             ('vpn', 'vpn-connection')]:
            item_id = fakes.random_ec2_id(r_id)
            self.db_api.get_tags.return_value = [{'item_id': item_id,
                                                  'key': 'fake-key',
                                                  'value': 'fake-value'}]
            resp = self.execute('DescribeTags', {})
            self.assertEqual({'tagSet': [{'resourceType': r_type,
                                          'resourceId': item_id,
                                          'key': 'fake-key',
                                          'value': 'fake-value'}]},
                             resp)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from unittest import mock

import fixtures
from glanceclient.common import exceptions as glance_exception
from oslo_config import fixture as config_fixture
import testtools

from ec2api.api import ec2utils
from ec2api import exception
from ec2api.tests.unit import base
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers


class EC2UtilsTestCase(testtools.TestCase):

    @mock.patch('ec2api.db.api.IMPL')
    def test_get_db_item(self, db_api):
        item = {'fake_key': 'fake_value'}
        db_api.get_item_by_id.return_value = item

        def check_normal_flow(kind, ec2_id):
            item['id'] = ec2_id
            res = ec2utils.get_db_item('fake_context', ec2_id)
            self.assertThat(res, matchers.DictMatches(item))
            db_api.get_item_by_id.assert_called_once_with('fake_context',
                                                          ec2_id)
            db_api.reset_mock()

        check_normal_flow('vpc', 'vpc-001234af')
        check_normal_flow('igw', 'igw-00000022')

        def check_not_found(kind, ex_class):
            ec2_id = fakes.random_ec2_id(kind)
            self.assertRaises(ex_class,
                              ec2utils.get_db_item,
                              'fake_context', ec2_id)
            db_api.get_item_by_id.assert_called_once_with('fake_context',
                                                          ec2_id)
            db_api.reset_mock()

        db_api.get_item_by_id.return_value = None
        check_not_found('vpc', exception.InvalidVpcIDNotFound)
        check_not_found('igw', exception.InvalidInternetGatewayIDNotFound)
        check_not_found('subnet', exception.InvalidSubnetIDNotFound)
        check_not_found('eni', exception.InvalidNetworkInterfaceIDNotFound)
        check_not_found('dopt', exception.InvalidDhcpOptionsIDNotFound)
        check_not_found('eipalloc', exception.InvalidAllocationIDNotFound)
        check_not_found('sg', exception.InvalidGroupNotFound)
        check_not_found('rtb', exception.InvalidRouteTableIDNotFound)
        check_not_found('i', exception.InvalidInstanceIDNotFound)
        check_not_found('vol', exception.InvalidVolumeNotFound)
        check_not_found('snap', exception.InvalidSnapshotNotFound)
        check_not_found('ami', exception.InvalidAMIIDNotFound)
        check_not_found('ari', exception.InvalidAMIIDNotFound)
        check_not_found('aki', exception.InvalidAMIIDNotFound)
        check_not_found('vgw', exception.InvalidVpnGatewayIDNotFound)
        check_not_found('cgw', exception.InvalidCustomerGatewayIDNotFound)
        check_not_found('vpn', exception.InvalidVpnConnectionIDNotFound)

    @mock.patch('ec2api.db.api.IMPL')
    def test_get_db_items(self, db_api):
        items = [{'id': fakes.random_ec2_id('fake'),
                  'fake_key': 'fake_value'},
                 {'id': fakes.random_ec2_id('fake'),
                  'fake_key': 'fake_value'}]
        db_api.get_items.return_value = items
        db_api.get_items_by_ids.return_value = items

        def check_with_no_filter(empty_filter):
            res = ec2utils.get_db_items('fake_context', 'fake', empty_filter)
            self.assertThat(res, matchers.ListMatches(items))
            db_api.get_items.assert_called_once_with('fake_context', 'fake')
            db_api.reset_mock()

        check_with_no_filter(None)
        check_with_no_filter([])

        def check_with_filter(item_ids):
            res = ec2utils.get_db_items('fake_context', 'fake', item_ids)
            self.assertThat(res, matchers.ListMatches(items))
            db_api.get_items_by_ids.assert_called_once_with(
                'fake_context', set(item_ids))
            db_api.reset_mock()

        item_ids = [i['id'] for i in items]
        check_with_filter(item_ids)
        check_with_filter(item_ids * 2)

        def check_not_found(kind, ex_class):
            items = [{'id': fakes.random_ec2_id(kind),
                      'fake_key': 'fake_value'} for _ in range(2)]
            item_ids = [i['id'] for i in items]
            item_ids.append(fakes.random_ec2_id(kind))
            db_api.get_items_by_ids.return_value = items
            self.assertRaises(ex_class, ec2utils.get_db_items,
                              'fake_context', kind, item_ids)
            db_api.reset_mock()

        check_not_found('vpc', exception.InvalidVpcIDNotFound)
        check_not_found('igw', exception.InvalidInternetGatewayIDNotFound)
        check_not_found('subnet', exception.InvalidSubnetIDNotFound)
        check_not_found('eni', exception.InvalidNetworkInterfaceIDNotFound)
        check_not_found('dopt', exception.InvalidDhcpOptionsIDNotFound)
        check_not_found('eipalloc', exception.InvalidAllocationIDNotFound)
        check_not_found('sg', exception.InvalidGroupNotFound)
        check_not_found('rtb', exception.InvalidRouteTableIDNotFound)
        check_not_found('i', exception.InvalidInstanceIDNotFound)
        check_not_found('vol', exception.InvalidVolumeNotFound)
        check_not_found('snap', exception.InvalidSnapshotNotFound)
        check_not_found('ami', exception.InvalidAMIIDNotFound)
        check_not_found('aki', exception.InvalidAMIIDNotFound)
        check_not_found('ari', exception.InvalidAMIIDNotFound)
        check_not_found('vgw', exception.InvalidVpnGatewayIDNotFound)
        check_not_found('cgw', exception.InvalidCustomerGatewayIDNotFound)
        check_not_found('vpn', exception.InvalidVpnConnectionIDNotFound)

    """Unit test api xml conversion."""
    def test_number_conversion(self):
        conv = ec2utils._try_convert
        self.assertIsNone(conv('None'))
        self.assertEqual(conv('True'), True)
        self.assertEqual(conv('TRUE'), True)
        self.assertEqual(conv('true'), True)
        self.assertEqual(conv('False'), False)
        self.assertEqual(conv('FALSE'), False)
        self.assertEqual(conv('false'), False)
        self.assertEqual(conv('0'), 0)
        self.assertEqual(conv('42'), 42)
        self.assertEqual(conv('3.14'), 3.14)
        self.assertEqual(conv('-57.12'), -57.12)
        self.assertEqual(conv('0x57'), 0x57)
        self.assertEqual(conv('-0x57'), -0x57)
        self.assertEqual(conv('-'), '-')
        self.assertEqual(conv('-0'), 0)
        self.assertEqual(conv('0.0'), 0.0)
        self.assertEqual(conv('1e-8'), 0.0)
        self.assertEqual(conv('-1e-8'), 0.0)
        self.assertEqual(conv('0xDD8G'), '0xDD8G')
        self.assertEqual(conv('0XDD8G'), '0XDD8G')
        self.assertEqual(conv('-stringy'), '-stringy')
        self.assertEqual(conv('stringy'), 'stringy')
        self.assertEqual(conv('add'), 'add')
        self.assertEqual(conv('remove'), 'remove')
        self.assertEqual(conv(''), '')

    @mock.patch('ec2api.db.api.IMPL')
    def test_os_id_to_ec2_id(self, db_api):
        fake_context = base.create_context()
        fake_id = fakes.random_ec2_id('fake')
        fake_os_id = fakes.random_os_id()

        # no cache, item is found
        db_api.get_items_ids.return_value = [(fake_id, fake_os_id)]
        item_id = ec2utils.os_id_to_ec2_id(fake_context, 'fake', fake_os_id)
        self.assertEqual(fake_id, item_id)
        db_api.get_items_ids.assert_called_once_with(
            fake_context, 'fake', item_ids=None, item_os_ids=(fake_os_id,))
        self.assertFalse(db_api.add_item_id.called)

        # no cache, item isn't found
        db_api.get_items_ids.return_value = []
        db_api.add_item_id.return_value = fake_id
        item_id = ec2utils.os_id_to_ec2_id(fake_context, 'fake', fake_os_id)
        self.assertEqual(fake_id, item_id)
        db_api.add_item_id.assert_called_once_with(
            fake_context, 'fake', fake_os_id, None)

        # no item in cache, item isn't found
        db_api.reset_mock()
        ids_cache = {fakes.random_os_id(): fakes.random_ec2_id('fake')}
        item_id = ec2utils.os_id_to_ec2_id(fake_context, 'fake', fake_os_id,
                                           ids_by_os_id=ids_cache)
        self.assertEqual(fake_id, item_id)
        self.assertIn(fake_os_id, ids_cache)
        self.assertEqual(fake_id, ids_cache[fake_os_id])
        db_api.add_item_id.assert_called_once_with(
            fake_context, 'fake', fake_os_id, None)

        # no item in cache, item is found
        db_api.reset_mock()
        db_api.get_items_ids.return_value = [(fake_id, fake_os_id)]
        ids_cache = {}
        item_id = ec2utils.os_id_to_ec2_id(fake_context, 'fake', fake_os_id,
                                           ids_by_os_id=ids_cache)
        self.assertEqual(fake_id, item_id)
        self.assertEqual({fake_os_id: fake_id}, ids_cache)
        self.assertFalse(db_api.add_item_id.called)

        # item in cache
        db_api.reset_mock()
        ids_cache = {fake_os_id: fake_id}
        item_id = ec2utils.os_id_to_ec2_id(fake_context, 'fake', fake_os_id,
                                           ids_by_os_id=ids_cache)
        self.assertEqual(fake_id, item_id)
        self.assertEqual({fake_os_id: fake_id}, ids_cache)
        self.assertFalse(db_api.get_items_ids.called)
        self.assertFalse(db_api.add_item_id.called)

        # item in items dict
        items_dict = {fake_os_id: {'id': fake_id,
                                   'os_id': fake_os_id}}
        ids_cache = {}
        item_id = ec2utils.os_id_to_ec2_id(fake_context, 'fake', fake_os_id,
                                           items_by_os_id=items_dict,
                                           ids_by_os_id=ids_cache)
        self.assertEqual(fake_id, item_id)
        self.assertFalse(db_api.get_items_ids.called)
        self.assertFalse(db_api.add_item_id.called)
        self.assertEqual({}, ids_cache)

        # item not in items dict, item is found
        items_dict = {fake_os_id: {'id': fake_id,
                                   'os_id': fake_os_id}}
        db_api.get_items_ids.return_value = [(fake_id, fake_os_id)]
        item_id = ec2utils.os_id_to_ec2_id(fake_context, 'fake', fake_os_id,
                                           items_by_os_id=items_dict)
        self.assertEqual(fake_id, item_id)
        self.assertFalse(db_api.add_item_id.called)

    @mock.patch('glanceclient.client.Client')
    @mock.patch('ec2api.db.api.IMPL')
    def test_get_os_image(self, db_api, glance):
        glance = glance.return_value
        fake_context = base.create_context()

        os_image = fakes.OSImage(fakes.OS_IMAGE_1)
        glance.images.get.return_value = os_image
        # check normal flow
        db_api.get_items_ids.return_value = [
            (fakes.ID_EC2_IMAGE_1, fakes.ID_OS_IMAGE_1)]
        self.assertEqual(
            os_image,
            ec2utils.get_os_image(fake_context, fakes.ID_EC2_IMAGE_1))
        db_api.get_items_ids.assert_called_with(
            mock.ANY, 'ami', item_ids=(fakes.ID_EC2_IMAGE_1,),
            item_os_ids=None)
        glance.images.get.assert_called_with(fakes.ID_OS_IMAGE_1)

        # check case of absence of an image in OS
        glance.images.get.side_effect = glance_exception.HTTPNotFound()
        self.assertRaises(
            exception.InvalidAMIIDNotFound,
            ec2utils.get_os_image,
            fake_context, fakes.ID_EC2_IMAGE_1)

        # check case of an unknown image id
        db_api.get_items_ids.return_value = []
        self.assertRaises(
            exception.InvalidAMIIDNotFound,
            ec2utils.get_os_image,
            fake_context, fakes.random_ec2_id('ami'))

        # check case of creating image
        db_api.get_items_ids.return_value = [(fakes.ID_EC2_IMAGE_1, None)]
        self.assertIsNone(ec2utils.get_os_image(fake_context,
                                                fakes.ID_EC2_IMAGE_1))

    @mock.patch('neutronclient.v2_0.client.Client')
    def test_get_os_public_network(self, neutron):
        neutron = neutron.return_value
        context = base.create_context()
        conf = self.useFixture(config_fixture.Config())

        conf.config(external_network='fake_public_network')
        neutron.list_networks.return_value = {'networks': ['network_object']}
        net = ec2utils.get_os_public_network(context)
        self.assertEqual('network_object', net)
        neutron.list_networks.assert_called_once_with(
            **{'router:external': True, 'name': 'fake_public_network'})

        neutron.list_networks.return_value = {'networks': []}
        with fixtures.FakeLogger() as log:
            self.assertRaises(exception.Unsupported,
                              ec2utils.get_os_public_network, context)
        self.assertNotEqual(0, len(log.output))
        self.assertIn('fake_public_network', log.output)

        neutron.list_networks.return_value = {'networks': ['obj1', 'obj2']}
        with fixtures.FakeLogger() as log:
            self.assertRaises(exception.Unsupported,
                              ec2utils.get_os_public_network, context)
        self.assertNotEqual(0, len(log.output))
        self.assertIn('fake_public_network', log.output)

        conf.config(external_network=None)
        with fixtures.FakeLogger() as log:
            self.assertRaises(exception.Unsupported,
                              ec2utils.get_os_public_network, context)
        self.assertNotEqual(0, len(log.output))
        self.assertNotIn('None', log.output)

        neutron.list_networks.return_value = {'networks': []}
        with fixtures.FakeLogger() as log:
            self.assertRaises(exception.Unsupported,
                              ec2utils.get_os_public_network, context)
        self.assertNotEqual(0, len(log.output))
        self.assertNotIn('None', log.output)

    def test_get_os_image_mappings(self):
        # check virtual device mapping transformation with substitution
        properties = {
            'mappings': [
                {'device': '/dev/vda', 'virtual': 'root'},
                {'device': 'vda', 'virtual': 'ami'},
                {'device': 'vdb', 'virtual': 'ephemeral0'},
                {'device': '/dev/vdb', 'virtual': 'swap'},
                {'device': '/dev/vdc', 'virtual': 'swap'},
                {'device': 'vdc', 'virtual': 'ephemeral0'},
                {'device': 'vdd'},
                {'device': '/dev/vdd', 'virtual': None},
                {'device': 'vdd', 'virtual': ''},
                {'device': '/dev/vdd', 'virtual': 'swamp'},
                {'virtual': 'ephemeral2'},
                {'device': None, 'virtual': 'ephemeral3'},
                {'device': '', 'virtual': 'ephemeral4'},
            ],
        }
        expected = [
            {'device_name': '/dev/vdb',
             'source_type': 'blank',
             'destination_type': 'local',
             'device_type': 'disk',
             'delete_on_termination': True,
             'boot_index': -1,
             'guest_format': 'swap',
             'virtual_name': 'swap'},
            {'device_name': '/dev/vdc',
             'source_type': 'blank',
             'destination_type': 'local',
             'device_type': 'disk',
             'delete_on_termination': True,
             'boot_index': -1,
             'virtual_name': 'ephemeral0'},
            {'device_name': None,
             'source_type': 'blank',
             'destination_type': 'local',
             'device_type': 'disk',
             'delete_on_termination': True,
             'boot_index': -1,
             'virtual_name': 'ephemeral2'},
            {'device_name': None,
             'source_type': 'blank',
             'destination_type': 'local',
             'device_type': 'disk',
             'delete_on_termination': True,
             'boot_index': -1,
             'virtual_name': 'ephemeral3'},
            {'device_name': '',
             'source_type': 'blank',
             'destination_type': 'local',
             'device_type': 'disk',
             'delete_on_termination': True,
             'boot_index': -1,
             'virtual_name': 'ephemeral4'},
        ]
        result = ec2utils.get_os_image_mappings(properties)
        self.assertThat(expected, matchers.ListMatches(result), verbose=True)

        # check legacy block device mapping transformation with substitution
        properties = {
            'block_device_mapping': [
                {'device_name': '/dev/vdb',
                 'virtual_name': 'ephemeral0'},
                {'device_name': 'vdc',
                 'virtual_name': 'swap',
                 'snapshot_id': 'fake_snapshot_id_0'},
                {'device_name': '/dev/vda',
                 'snapshot_id': 'fake_snapshot_id_1',
                 'delete_on_termination': True,
                 'volume_size': 100},
                {'snapshot_id': 'fake_snapshot_id_2'},
                {'device_name': '/dev/vdd',
                 'virtual_name': 'ephemeral2'},
                {'device_name': 'vdd',
                 'volume_id': 'fake_volume_id_3',
                 'delete_on_termination': False},
                {'device_name': 'vde',
                 'volume_id': 'fake_volume_id_4'},
                {'device_name': '/dev/vde',
                 'snapshot_id': 'fake_snapshot_id_4',
                 'no_device': True},
                {'snapshot_id': 'fake_snapshot_id_5',
                 'volume_id': 'fake_volume_id_5',
                 'volume_size': 50},
            ],
        }
        expected = [
            {'device_name': '/dev/vdb',
             'source_type': 'blank',
             'destination_type': 'local',
             'device_type': 'disk',
             'delete_on_termination': True,
             'boot_index': -1,
             'virtual_name': 'ephemeral0'},
            {'device_name': 'vdc',
             'source_type': 'blank',
             'destination_type': 'local',
             'device_type': 'disk',
             'delete_on_termination': True,
             'boot_index': -1,
             'guest_format': 'swap',
             'virtual_name': 'swap'},
            {'device_name': '/dev/vda',
             'snapshot_id': 'fake_snapshot_id_1',
             'source_type': 'snapshot',
             'destination_type': 'volume',
             'delete_on_termination': True,
             'volume_size': 100},
            {'snapshot_id': 'fake_snapshot_id_2',
             'source_type': 'snapshot',
             'destination_type': 'volume',
             'delete_on_termination': False},
            {'device_name': 'vdd',
             'volume_id': 'fake_volume_id_3',
             'source_type': 'volume',
             'destination_type': 'volume',
             'delete_on_termination': False},
            {'device_name': '/dev/vde',
             'snapshot_id': 'fake_snapshot_id_4',
             'source_type': 'snapshot',
             'destination_type': 'volume',
             'no_device': True,
             'delete_on_termination': False},
            {'snapshot_id': 'fake_snapshot_id_5',
             'volume_id': 'fake_volume_id_5',
             'source_type': 'snapshot',
             'destination_type': 'volume',
             'volume_size': 50,
             'delete_on_termination': False},
        ]
        result = ec2utils.get_os_image_mappings(properties)
        self.assertThat(expected, matchers.ListMatches(result), verbose=True)

        # check bdm v2 with substitution
        properties = {
            'bdm_v2': True,
            'block_device_mapping': [
                {'device_name': '/dev/vdb',
                 'snapshot_id': 'fake_snapshot_id_1',
                 'source_type': 'snapshot',
                 'destination_type': 'volume',
                 'volume_size': 20,
                 'delete_on_termination': True},
                {'device_name': '/dev/vdb',
                 'source_type': 'blank',
                 'destination_type': 'volume',
                 'volume_size': 10,
                 'delete_on_termination': True},
                {'device_name': '/dev/vdc',
                 'snapshot_id': 'fake_snapshot_id_2',
                 'source_type': 'snapshot',
                 'destination_type': 'volume'},
                {'device_name': 'vdc',
                 'volume_id': 'fake_volume_id_2',
                 'source_type': 'volume',
                 'destination_type': 'volume'},
                {'device_name': 'vdd',
                 'snapshot_id': 'fake_snapshot_id_3',
                 'source_type': 'snapshot',
                 'destination_type': 'volume'},
                {'device_name': '/dev/vdd',
                 'image_id': 'fake_image_id_1',
                 'source_type': 'image',
                 'destination_type': 'volume',
                 'volume_size': 30},
            ],
        }
        expected = [
            {'device_name': '/dev/vdb',
             'source_type': 'blank',
             'destination_type': 'volume',
             'volume_size': 10,
             'delete_on_termination': True},
            {'device_name': 'vdc',
             'volume_id': 'fake_volume_id_2',
             'source_type': 'volume',
             'destination_type': 'volume',
             'delete_on_termination': False},
            {'device_name': '/dev/vdd',
             'image_id': 'fake_image_id_1',
             'source_type': 'image',
             'destination_type': 'volume',
             'volume_size': 30,
             'delete_on_termination': False},
        ]
        result = ec2utils.get_os_image_mappings(properties)
        self.assertThat(expected, matchers.ListMatches(result), verbose=True)

        # check bdm v2 vs vdm susbtitution
        properties = {
            'mappings': [
                {'device': 'vdb', 'virtual': 'ephemeral0'},
                {'device': 'vdc', 'virtual': 'ephemeral1'},
                {'device': 'vdh', 'virtual': 'ephemeral2'},
            ],
            'bdm_v2': True,
            'block_device_mapping': [
                {'device_name': '/dev/vda',
                 'snapshot_id': 'fake_snapshot_id_1',
                 'source_type': 'snapshot',
                 'destination_type': 'volume'},
                {'device_name': '/dev/vdc',
                 'snapshot_id': 'fake_snapshot_id_2',
                 'source_type': 'snapshot',
                 'destination_type': 'volume'},
                {'device_name': '/dev/vdd',
                 'snapshot_id': 'fake_snapshot_id_3',
                 'source_type': 'snapshot',
                 'destination_type': 'volume'}
            ],
        }
        expected = [
            {'device_name': '/dev/vdb',
             'source_type': 'blank',
             'destination_type': 'local',
             'device_type': 'disk',
             'delete_on_termination': True,
             'boot_index': -1,
             'virtual_name': 'ephemeral0'},
            {'device_name': '/dev/vdc',
             'snapshot_id': 'fake_snapshot_id_2',
             'source_type': 'snapshot',
             'destination_type': 'volume',
             'delete_on_termination': False},
            {'device_name': '/dev/vdh',
             'source_type': 'blank',
             'destination_type': 'local',
             'device_type': 'disk',
             'delete_on_termination': True,
             'boot_index': -1,
             'virtual_name': 'ephemeral2'},
            {'device_name': '/dev/vda',
             'snapshot_id': 'fake_snapshot_id_1',
             'source_type': 'snapshot',
             'destination_type': 'volume',
             'delete_on_termination': False},
            {'device_name': '/dev/vdd',
             'snapshot_id': 'fake_snapshot_id_3',
             'source_type': 'snapshot',
             'destination_type': 'volume',
             'delete_on_termination': False},
        ]
        result = ec2utils.get_os_image_mappings(properties)
        self.assertThat(expected, matchers.ListMatches(result), verbose=True)

        # check legacy bdm vs vdm susbtitution
        properties = {
            'mappings': [
                {'device': 'vdb', 'virtual': 'ephemeral0'},
                {'device': 'vdc', 'virtual': 'ephemeral1'},
                {'device': 'vdh', 'virtual': 'ephemeral2'},
            ],
            'block_device_mapping': [
                {'device_name': '/dev/vda',
                 'snapshot_id': 'fake_snapshot_id_1'},
                {'device_name': '/dev/vdc',
                 'snapshot_id': 'fake_snapshot_id_2'},
                {'device_name': '/dev/vdd',
                 'snapshot_id': 'fake_snapshot_id_3'}
            ],
        }
        result = ec2utils.get_os_image_mappings(properties)
        self.assertThat(expected, matchers.ListMatches(result), verbose=True)

    def test_block_device_strip_dev(self):
        self.assertEqual(ec2utils.block_device_strip_dev('/dev/sda'), 'sda')
        self.assertEqual(ec2utils.block_device_strip_dev('sda'), 'sda')

    def test_block_device_prepend_dev(self):
        mapping = ['/dev/sda', 'sdb', 'sdc', 'sdd', 'sde']
        expected = ['/dev/sda', '/dev/sdb', '/dev/sdc', '/dev/sdd', '/dev/sde']

        for m, e in zip(mapping, expected):
            prepended = ec2utils.block_device_prepend_dev(m)
            self.assertEqual(e, prepended)

    def test_block_device_properties_root_device_name(self):
        root_device0 = '/dev/sda'
        root_device1 = '/dev/sdb'
        mappings = [{'virtual': 'root',
                     'device': root_device0}]

        properties0 = {'mappings': mappings}
        properties1 = {'mappings': mappings,
                       'root_device_name': root_device1}

        self.assertIsNone(
            ec2utils.block_device_properties_root_device_name({}))
        self.assertEqual(
            root_device0,
            ec2utils.block_device_properties_root_device_name(properties0))
        self.assertEqual(
            root_device1,
            ec2utils.block_device_properties_root_device_name(properties1))



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
:mod:`ec2api.tests.unit.unit` -- EC2api Unittests
=====================================================

.. automodule:: ec2api.tests.unit.unit
   :platform: Unix
"""

# See http://code.google.com/p/python-nose/issues/detail?id=373
# The code below enables nosetests to work with i18n _() blocks
from six.moves import builtins
setattr(builtins, '_', lambda x: x)

# NOTE(ft): this is required by test_s3.S3APITestCase to switch execution
# between test and server threads
import eventlet  # noqa: E402
eventlet.monkey_patch(socket=True)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from oslo_config import cfg

path_opts = [
    cfg.StrOpt('state_path',
               default='/var/lib/ec2api',
               help="Top-level directory for maintaining ec2api's state"),
]

CONF = cfg.CONF
CONF.register_opts(path_opts)


def state_path_def(*args):
    """Return an uninterpolated path relative to $state_path."""
    return os.path.join('$state_path', *args)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import re

from glanceclient.common import exceptions as glance_exception
from lxml import etree
from oslo_config import cfg
from oslo_log import log as logging
from oslo_serialization import jsonutils
from oslo_utils import timeutils
import six

from ec2api import clients
from ec2api.db import api as db_api
from ec2api import exception
from ec2api.i18n import _

LOG = logging.getLogger(__name__)

ec2_opts = [
    cfg.StrOpt('external_network',
               default=None,
               help='Name of the external network, which is used to connect'
                    'VPCs to Internet and to allocate Elastic IPs.'),
]

CONF = cfg.CONF
CONF.register_opts(ec2_opts)

LEGACY_BDM_FIELDS = set(['device_name', 'delete_on_termination', 'snapshot_id',
                         'volume_id', 'volume_size', 'no_device'])

_c2u = re.compile('(((?<=[a-z])[A-Z])|([A-Z](?![A-Z]|$)))')


def camelcase_to_underscore(str):
    return _c2u.sub(r'_\1', str).lower().strip('_')


def _try_convert(value):
    """Return a non-string from a string or unicode, if possible.

    ============= =====================================================
    When value is returns
    ============= =====================================================
    zero-length   ''
    'None'        None
    'True'        True case insensitive
    'False'       False case insensitive
    '0', '-0'     0
    0xN, -0xN     int from hex (positive) (N is any number)
    0bN, -0bN     int from binary (positive) (N is any number)
    *             try conversion to int, float, complex, fallback value

    """
    def _negative_zero(value):
        epsilon = 1e-7
        return 0 if abs(value) < epsilon else value

    if len(value) == 0:
        return ''
    if value == 'None':
        return None
    lowered_value = value.lower()
    if lowered_value == 'true':
        return True
    if lowered_value == 'false':
        return False
    for prefix, base in [('0x', 16), ('0b', 2), ('0', 8), ('', 10)]:
        try:
            if lowered_value.startswith((prefix, "-" + prefix)):
                return int(lowered_value, base)
        except ValueError:
            pass
    try:
        return _negative_zero(float(value))
    except ValueError:
        return value


def dict_from_dotted_str(items):
    """parse multi dot-separated argument into dict.

    EBS boot uses multi dot-separated arguments like
    BlockDeviceMapping.1.DeviceName=snap-id
    Convert the above into
    {'block_device_mapping': {'1': {'device_name': snap-id}}}
    """
    args = {}
    for key, value in items:
        parts = key.split(".")
        key = str(camelcase_to_underscore(parts[0]))
        if isinstance(value, six.string_types):
            # NOTE(vish): Automatically convert strings back
            #             into their respective values
            value = _try_convert(value)

            if len(parts) > 1:
                d = args.get(key, {})
                args[key] = d
                for k in parts[1:-1]:
                    k = camelcase_to_underscore(k)
                    v = d.get(k, {})
                    d[k] = v
                    d = v
                d[camelcase_to_underscore(parts[-1])] = value
            else:
                args[key] = value

    return args


def _render_dict(el, data):
    try:
        for key, val in data.items():
            sub_el = etree.SubElement(el, key)
            _render_data(sub_el, val)
    except Exception:
        LOG.debug(data)
        raise


def _render_data(el, data):
    if isinstance(data, list):
        for item in data:
            sub_el = etree.SubElement(el, 'item')
            _render_data(sub_el, item)
    elif isinstance(data, dict):
        _render_dict(el, data)
    elif hasattr(data, '__dict__'):
        _render_dict(el, data.__dict__)
    elif isinstance(data, bool):
        el.text = str(data).lower()
    elif isinstance(data, datetime.datetime):
        el.text = _database_to_isoformat(data)
    elif isinstance(data, six.binary_type):
        el.text = data.decode("utf-8")
    elif data is not None:
        el.text = six.text_type(data)


def _database_to_isoformat(datetimeobj):
    """Return a xs:dateTime parsable string from datatime."""
    return datetimeobj.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + 'Z'


def dict_to_xml(data_dict, root_tag):
    root = etree.Element(root_tag)
    _render_dict(root, data_dict)
    return root


_ms_time_regex = re.compile(r'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3,6}Z$')


def is_ec2_timestamp_expired(request, expires=None):
    """Checks the timestamp or expiry time included in an EC2 request

    and returns true if the request is expired
    """
    query_time = None
    timestamp = request.get('Timestamp')
    expiry_time = request.get('Expires')

    def parse_strtime(strtime):
        if _ms_time_regex.match(strtime):
            # NOTE(MotoKen): time format for aws-sdk-java contains millisecond
            time_format = "%Y-%m-%dT%H:%M:%S.%fZ"
        else:
            time_format = "%Y-%m-%dT%H:%M:%SZ"
        return timeutils.parse_strtime(strtime, time_format)

    try:
        if timestamp and expiry_time:
            msg = _("Request must include either Timestamp or Expires,"
                    " but cannot contain both")
            LOG.error(msg)
            raise exception.InvalidRequest(msg)
        elif expiry_time:
            query_time = parse_strtime(expiry_time)
            return timeutils.is_older_than(query_time, -1)
        elif timestamp:
            query_time = parse_strtime(timestamp)

            # Check if the difference between the timestamp in the request
            # and the time on our servers is larger than 5 minutes, the
            # request is too old (or too new).
            if query_time and expires:
                return (timeutils.is_older_than(query_time, expires) or
                        timeutils.is_newer_than(query_time, expires))
        return False
    except ValueError:
        LOG.exception("Timestamp is invalid: ")
        return True


# NOTE(ft): extra functions to use in vpc specific code or instead of
# malformed existed functions


def get_ec2_id_kind(obj_id):
    return obj_id.split('-')[0]


def change_ec2_id_kind(obj_id, new_kind):
    return '%(kind)s-%(id)s' % {'kind': new_kind,
                                'id': obj_id.split('-')[-1]}


NOT_FOUND_EXCEPTION_MAP = {
    'vpc': exception.InvalidVpcIDNotFound,
    'igw': exception.InvalidInternetGatewayIDNotFound,
    'subnet': exception.InvalidSubnetIDNotFound,
    'eni': exception.InvalidNetworkInterfaceIDNotFound,
    'dopt': exception.InvalidDhcpOptionsIDNotFound,
    'eipalloc': exception.InvalidAllocationIDNotFound,
    'sg': exception.InvalidGroupNotFound,
    'rtb': exception.InvalidRouteTableIDNotFound,
    'i': exception.InvalidInstanceIDNotFound,
    'kp': exception.InvalidKeypairNotFound,
    'az': exception.InvalidAvailabilityZoneNotFound,
    'vol': exception.InvalidVolumeNotFound,
    'snap': exception.InvalidSnapshotNotFound,
    'ami': exception.InvalidAMIIDNotFound,
    'aki': exception.InvalidAMIIDNotFound,
    'ari': exception.InvalidAMIIDNotFound,
    'vgw': exception.InvalidVpnGatewayIDNotFound,
    'cgw': exception.InvalidCustomerGatewayIDNotFound,
    'vpn': exception.InvalidVpnConnectionIDNotFound,
}


def get_db_item(context, ec2_id, expected_kind=None):
    """Get an DB item, raise AWS compliant exception if it's not found.

        Args:
            context (RequestContext): The request context.
            ec2_id (str): The ID of the requested item.
            expected_kind (str): The expected kind of the requested item.
                It should be specified for a kind of ec2_id to be validated,
                if you need it.

        Returns:
            The DB item.
    """
    item = db_api.get_item_by_id(context, ec2_id)
    if (item is None or
            expected_kind and get_ec2_id_kind(ec2_id) != expected_kind):
        kind = expected_kind or get_ec2_id_kind(ec2_id)
        params = {'id': ec2_id}
        raise NOT_FOUND_EXCEPTION_MAP[kind](**params)
    return item


def get_db_items(context, kind, ec2_ids):
    if not ec2_ids:
        return db_api.get_items(context, kind)

    if not isinstance(ec2_ids, set):
        ec2_ids = set(ec2_ids)
    items = db_api.get_items_by_ids(context, ec2_ids)
    if len(items) < len(ec2_ids):
        missed_ids = ec2_ids - set((item['id'] for item in items))
        params = {'id': next(iter(missed_ids))}
        raise NOT_FOUND_EXCEPTION_MAP[kind](**params)
    return items


_auto_create_db_item_extensions = {}


def register_auto_create_db_item_extension(kind, extension):
    _auto_create_db_item_extensions[kind] = extension


def auto_create_db_item(context, kind, os_id, **extension_kwargs):
    item = {'os_id': os_id}
    extension = _auto_create_db_item_extensions.get(kind)
    if extension:
        extension(context, item, **extension_kwargs)
    return db_api.add_item(context, kind, item)


def get_db_item_by_os_id(context, kind, os_id, items_by_os_id=None,
                         **extension_kwargs):
    """Get DB item by OS id (create if it doesn't exist).

        Args:
            context (RequestContext): The request context.
            kind (str): The kind of item.
            os_id (str): OS id of an object.
            items_by_os_id (dict of items): The dict of known DB items,
                OS id is used as a key.
            extension_kwargs (dict): Additional parameters passed to
                a registered extension at creating item.

        Returns:
            A found or created item.

        Search item in passed dict. If it's not found - create a new item, and
        add it to the dict (if it's passed).
        If an extension is registered on corresponding item kind, call it
        passing extension_kwargs to it.
    """
    if os_id is None:
        return None
    if items_by_os_id is not None:
        item = items_by_os_id.get(os_id)
        if item:
            return item
    else:
        item = next((i for i in db_api.get_items(context, kind)
                     if i['os_id'] == os_id), None)
    if not item:
        item = auto_create_db_item(context, kind, os_id, **extension_kwargs)
    if items_by_os_id is not None:
        items_by_os_id[os_id] = item
    return item


# TODO(Alex): The project_id passing mechanism can be potentially
# reconsidered in future.
def os_id_to_ec2_id(context, kind, os_id, items_by_os_id=None,
                    ids_by_os_id=None, project_id=None):
    if os_id is None:
        return None
    if ids_by_os_id is not None:
        item_id = ids_by_os_id.get(os_id)
        if item_id:
            return item_id
    if items_by_os_id is not None:
        item = items_by_os_id.get(os_id)
        if item:
            return item['id']
    ids = db_api.get_items_ids(context, kind, item_os_ids=(os_id,))
    if len(ids):
        item_id, _os_id = ids[0]
    else:
        item_id = db_api.add_item_id(context, kind, os_id,
                                     project_id=project_id)
    if ids_by_os_id is not None:
        ids_by_os_id[os_id] = item_id
    return item_id


def get_os_image(context, ec2_image_id):
    kind = get_ec2_id_kind(ec2_image_id)
    ids = db_api.get_items_ids(context, kind, item_ids=(ec2_image_id,))
    if not ids:
        raise exception.InvalidAMIIDNotFound(id=ec2_image_id)
    _id, os_id = ids[0]
    if not os_id:
        return None
    glance = clients.glance(context)
    try:
        return glance.images.get(os_id)
    except glance_exception.HTTPNotFound:
        raise exception.InvalidAMIIDNotFound(id=ec2_image_id)


def deserialize_os_image_properties(os_image):
    def prepare_property(property_name):
        if property_name in os_image_dict:
            os_image_dict[property_name] = jsonutils.loads(
                os_image_dict[property_name])

    os_image_dict = dict(os_image)
    prepare_property('mappings')
    prepare_property('block_device_mapping')
    return os_image_dict


def create_virtual_bdm(device_name, virtual_name):
    bdm = {'device_name': device_name,
           'source_type': 'blank',
           'destination_type': 'local',
           'device_type': 'disk',
           'delete_on_termination': True,
           'boot_index': -1,
           'virtual_name': virtual_name}
    if virtual_name == 'swap':
        bdm['guest_format'] = 'swap'
    return bdm


def get_os_image_mappings(os_image_properties):
    mappings = []
    names = set()
    # TODO(ft): validate device names for both virtual and block device
    # mappings

    def is_virtual(virtual_name):
        return virtual_name == 'swap' or (virtual_name and
                                          _ephemeral.match(virtual_name))

    # NOTE(ft): substitute mapping if the same device name is specified
    def add_mapping(mapping):
        device_name = block_device_strip_dev(mapping.get('device_name'))
        if device_name in names:
            for i, m in enumerate(mappings):
                if (device_name ==
                        block_device_strip_dev(m.get('device_name'))):
                    mappings[i] = mapping
                    break
        else:
            if device_name:
                names.add(device_name)
            mappings.append(mapping)

    # TODO(ft): From Juno virtual device mapping has precedence of block one
    # in boot logic. This function should do the same, despite Nova EC2
    # behavior.

    # NOTE(ft): Nova EC2 prepended device names for virtual device mappings.
    # But AWS doesn't do it.
    for vdm in os_image_properties.get('mappings', []):
        if is_virtual(vdm.get('virtual')):
            add_mapping(create_virtual_bdm(
                block_device_prepend_dev(vdm.get('device')), vdm['virtual']))

    legacy_mapping = not os_image_properties.get('bdm_v2', False)
    for bdm in os_image_properties.get('block_device_mapping', []):
        if legacy_mapping:
            virtual_name = bdm.get('virtual_name')
            if is_virtual(virtual_name):
                new_bdm = create_virtual_bdm(bdm.get('device_name'),
                                             virtual_name)
            else:
                new_bdm = {key: val for key, val in bdm.items()
                           if key in LEGACY_BDM_FIELDS}
                if bdm.get('snapshot_id'):
                    new_bdm.update({'source_type': 'snapshot',
                                    'destination_type': 'volume'})
                elif bdm.get('volume_id'):
                    new_bdm.update({'source_type': 'volume',
                                    'destination_type': 'volume'})
            bdm = new_bdm

        bdm.setdefault('delete_on_termination', False)
        add_mapping(bdm)

    return mappings


def get_os_public_network(context):
    neutron = clients.neutron(context)
    search_opts = {'router:external': True, 'name': CONF.external_network}
    os_networks = neutron.list_networks(**search_opts)['networks']
    if len(os_networks) != 1:
        if CONF.external_network:
            if len(os_networks) == 0:
                msg = "No external network with name '%s' is found"
            else:
                msg = "More than one external network with name '%s' is found"
            LOG.error(msg, CONF.external_network)
        else:
            if len(os_networks) == 0:
                msg = 'No external network is found'
            else:
                msg = 'More than one external network is found'
            LOG.error(msg)
        raise exception.Unsupported(_('Feature is restricted by OS admin'))
    return os_networks[0]


def get_attached_gateway(context, vpc_id, gateway_kind):
    # TODO(ft): move search by vpc_id to DB api
    return next((gw for gw in db_api.get_items(context, gateway_kind)
                 if gw['vpc_id'] == vpc_id), None)


_check_and_create_default_vpc = None


def check_and_create_default_vpc(context):
    return _check_and_create_default_vpc(context)


def set_check_and_create_default_vpc(check_and_create_default_vpc):
    global _check_and_create_default_vpc
    _check_and_create_default_vpc = check_and_create_default_vpc


def get_default_vpc(context):
    default_vpc = check_and_create_default_vpc(context)
    if not default_vpc:
        raise exception.VPCIdNotSpecified()
    return default_vpc


# NOTE(ft): following functions are copied from various parts of Nova

_ephemeral = re.compile(r'^ephemeral(\d|[1-9]\d+)$')

_dev = re.compile(r'^/dev/')


def block_device_strip_dev(device_name):
    """remove leading '/dev/'."""
    return _dev.sub('', device_name) if device_name else device_name


def block_device_prepend_dev(device_name):
    """Make sure there is a leading '/dev/'."""
    return device_name and '/dev/' + block_device_strip_dev(device_name)


def block_device_properties_root_device_name(properties):
    """get root device name from image meta data.

    If it isn't specified, return None.
    """
    if 'root_device_name' in properties:
        return properties.get('root_device_name')
    elif 'mappings' in properties:
        return next((bdm['device'] for bdm in properties['mappings']
                     if bdm['virtual'] == 'root'), None)
    else:
        return None


_ISO8601_TIME_FORMAT_SUBSECOND = '%Y-%m-%dT%H:%M:%S.%f'
_ISO8601_TIME_FORMAT = '%Y-%m-%dT%H:%M:%S'


def isotime(at=None, subsecond=False):
    """Stringify time in ISO 8601 format."""

    # Python provides a similar instance method for datetime.datetime objects
    # called isoformat(). The format of the strings generated by isoformat()
    # have a couple of problems:
    # 1) The strings generated by isotime are used in tokens and other public
    #    APIs that we can't change without a deprecation period. The strings
    #    generated by isoformat are not the same format, so we can't just
    #    change to it.
    # 2) The strings generated by isoformat do not include the microseconds if
    #    the value happens to be 0. This will likely show up as random failures
    #    as parsers may be written to always expect microseconds, and it will
    #    parse correctly most of the time.

    if not at:
        at = timeutils.utcnow()
    st = at.strftime(_ISO8601_TIME_FORMAT
                     if not subsecond
                     else _ISO8601_TIME_FORMAT_SUBSECOND)
    tz = at.tzinfo.tzname(None) if at.tzinfo else 'UTC'
    st += ('Z' if tz == 'UTC' else tz)
    return st
